@@ -599,7 +599,7 @@ static void InitGlobalAnimControls(void)
 	}
 
 	// apply special settings to pointer-style animations
-	if (part->control_info.class == get_hash_from_key("pointer"))
+	if (isClass(part->control_info.class, "pointer"))
 	{
 	  // force pointer-style animations to be checked for clicks first
 	  part->control_info.draw_order = 1000000;
@@ -671,7 +671,6 @@ static void SetGlobalAnimEventsForCustomElements(int list_pos)
 void InitGlobalAnimEventsForCustomElements(void)
 {
   int m, a, p;
-  int control;
 
   // custom element events for global animations only relevant while playing
   m = GAME_MODE_PLAYING;
@@ -679,8 +678,7 @@ void InitGlobalAnimEventsForCustomElements(void)
   for (a = 0; a < NUM_GLOBAL_ANIMS; a++)
   {
     int ctrl_id = GLOBAL_ANIM_ID_CONTROL_FIRST + a;
-
-    control = global_anim_info[ctrl_id].graphic[GLOBAL_ANIM_ID_PART_BASE][m];
+    int control = global_anim_info[ctrl_id].graphic[GLOBAL_ANIM_ID_PART_BASE][m];
 
     // if no base animation parameters defined, use default values
     if (control == IMG_UNDEFINED)
@@ -720,7 +718,7 @@ static void BlitGlobalAnimation(struct GlobalAnimPartControlInfo *part,
   Bitmap *fade_bitmap =
     (drawing_target == DRAW_TO_FADE_SOURCE ? gfx.fade_bitmap_source :
      drawing_target == DRAW_TO_FADE_TARGET ? gfx.fade_bitmap_target : NULL);
-  int alpha = (c->fade_mode & FADE_MODE_FADE ? part->fade_alpha : g->alpha);
+  int alpha = ((c->fade_mode & FADE_MODE_FADE) != 0 ? part->fade_alpha : g->alpha);
   int x, y;
 
   for (y = 0; y < c->stacked_yfactor; y++)
@@ -801,7 +799,6 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
     boolean before_fading = (global.anim_status == GAME_MODE_PSEUDO_FADING);
     boolean after_fading  = (anim_status_last   == GAME_MODE_PSEUDO_FADING);
     int anim_classes_next = game_mode_anim_classes[global.anim_status_next];
-    int i;
 
     if (drawing_target == DRAW_TO_FADE_TARGET)
       after_fading = TRUE;
@@ -987,7 +984,7 @@ static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part
   boolean changed = FALSE;
 
   if (part->last_anim_status == global.anim_status &&
-      part->control_info.class != get_hash_from_key("pointer"))
+      !isClass(part->control_info.class, "pointer"))
     return FALSE;
 
   part->last_anim_status = global.anim_status;
@@ -996,8 +993,8 @@ static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part
 
   part->class_playfield_or_door = FALSE;
 
-  if (part->control_info.class == get_hash_from_key("window") ||
-      part->control_info.class == get_hash_from_key("border"))
+  if (isClass(part->control_info.class, "window") ||
+      isClass(part->control_info.class, "border"))
   {
     viewport_x = 0;
     viewport_y = 0;
@@ -1006,7 +1003,7 @@ static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part
 
     part->drawing_stage = DRAW_GLOBAL_ANIM_STAGE_2;
   }
-  else if (part->control_info.class == get_hash_from_key("pointer"))
+  else if (isClass(part->control_info.class, "pointer"))
   {
     int mx = MIN(MAX(0, gfx.mouse_x), WIN_XSIZE - 1);
     int my = MIN(MAX(0, gfx.mouse_y), WIN_YSIZE - 1);
@@ -1027,7 +1024,7 @@ static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part
     if (global.anim_status != GAME_MODE_LOADING)
       gfx.cursor_mode_override = CURSOR_NONE;
   }
-  else if (part->control_info.class == get_hash_from_key("door_1"))
+  else if (isClass(part->control_info.class, "door_1"))
   {
     viewport_x = DX;
     viewport_y = DY;
@@ -1036,7 +1033,7 @@ static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part
 
     part->class_playfield_or_door = TRUE;
   }
-  else if (part->control_info.class == get_hash_from_key("door_2"))
+  else if (isClass(part->control_info.class, "door_2"))
   {
     if (part->mode_nr == GAME_MODE_EDITOR)
     {
@@ -1075,7 +1072,7 @@ static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part
     part->viewport_width  = viewport_width;
     part->viewport_height = viewport_height;
 
-    if (part->control_info.class != get_hash_from_key("pointer"))
+    if (!isClass(part->control_info.class, "pointer"))
       changed = TRUE;
   }
 
@@ -1613,7 +1610,7 @@ static int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part,
       part->step_yoffset = 0;
     }
 
-    if (part->control_info.class != get_hash_from_key("pointer"))
+    if (!isClass(part->control_info.class, "pointer"))
     {
       if (c->x != ARG_UNDEFINED_VALUE)
 	part->x = c->x;
@@ -2076,7 +2073,7 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, int clicked_event)
 
     // if request dialog is active, only handle pointer-style animations
     if (game.request_active &&
-	part->control_info.class != get_hash_from_key("pointer"))
+	!isClass(part->control_info.class, "pointer"))
       continue;
 
     if (clicked_event == ANIM_CLICKED_RESET)
@@ -2185,14 +2182,14 @@ static void ResetGlobalAnim_Clicked(void)
 
 void RestartGlobalAnimsByStatus(int status)
 {
-  int anim_status_last = global.anim_status;
+  int global_anim_status_last = global.anim_status;
 
   global.anim_status = status;
 
   // force restarting global animations by changed global animation status
   DrawGlobalAnimationsExt(DRAW_TO_SCREEN, DRAW_GLOBAL_ANIM_STAGE_RESTART);
 
-  global.anim_status = anim_status_last;
+  global.anim_status = global_anim_status_last;
 }
 
 void SetAnimStatusBeforeFading(int status)
