@@ -14,6 +14,8 @@
 
 #include "main.h"
 
+boolean isLevelEditorFastStart(void);
+
 void CreateLevelEditorGadgets(void);
 void FreeLevelEditorGadgets(void);
 void UnmapLevelEditorGadgets(void);
@@ -34,4 +36,9 @@ void CopyBrushToClipboard_Small(void);
 void UndoLevelEditorOperation(void);
 void RedoLevelEditorOperation(void);
 
+void SetDefaultLevelColorType_BD(void);
+void SetDefaultLevelColors_BD(void);
+void SetCurrentLevelColors_BD(void);
+void SetRandomLevelColors_BD(int);
+
 #endif
