@@ -21,6 +21,7 @@
 #include "tape.h"
 #include "anim.h"
 #include "network.h"
+#include "config.h"
 
 
 #define	DEBUG_EVENTS		0
@@ -562,10 +563,27 @@ void HandleMotionEvent(MotionEvent *event)
   HandleButton(event->x, event->y, button_status, button_status);
 }
 
+static void HandleWheelEventButton(int button_nr, int steps)
+{
+#if defined(PLATFORM_WINDOWS) || defined(PLATFORM_MAC)
+  // accelerated mouse wheel available on Mac and Windows
+  wheel_steps = steps;
+#else
+  // no accelerated mouse wheel available on Unix/Linux
+  wheel_steps = DEFAULT_WHEEL_STEPS;
+#endif
+
+  motion_status = FALSE;
+
+  button_status = button_nr;
+  HandleButton(0, 0, button_status, -button_nr);
+
+  button_status = MB_RELEASED;
+  HandleButton(0, 0, button_status, -button_nr);
+}
+
 void HandleWheelEvent(WheelEvent *event)
 {
-  int button_nr;
-
 #if DEBUG_EVENTS_WHEEL
 #if 1
   Debug("event:wheel", "mouse == %d, x/y == %d/%d\n",
@@ -579,26 +597,11 @@ void HandleWheelEvent(WheelEvent *event)
 #endif
 #endif
 
-  button_nr = (event->x < 0 ? MB_WHEEL_LEFT :
-	       event->x > 0 ? MB_WHEEL_RIGHT :
-	       event->y < 0 ? MB_WHEEL_DOWN :
-	       event->y > 0 ? MB_WHEEL_UP : 0);
+  if (event->x != 0)
+    HandleWheelEventButton(event->x < 0 ? MB_WHEEL_LEFT : MB_WHEEL_RIGHT, ABS(event->x));
 
-#if defined(PLATFORM_WINDOWS) || defined(PLATFORM_MAC)
-  // accelerated mouse wheel available on Mac and Windows
-  wheel_steps = (event->x ? ABS(event->x) : ABS(event->y));
-#else
-  // no accelerated mouse wheel available on Unix/Linux
-  wheel_steps = DEFAULT_WHEEL_STEPS;
-#endif
-
-  motion_status = FALSE;
-
-  button_status = button_nr;
-  HandleButton(0, 0, button_status, -button_nr);
-
-  button_status = MB_RELEASED;
-  HandleButton(0, 0, button_status, -button_nr);
+  if (event->y != 0)
+    HandleWheelEventButton(event->y < 0 ? MB_WHEEL_DOWN : MB_WHEEL_UP, ABS(event->y));
 }
 
 void HandleWindowEvent(WindowEvent *event)
@@ -666,6 +669,10 @@ void HandleWindowEvent(WindowEvent *event)
 	UpdateMousePosition();
 
 	SetWindowTitle();
+	SetWindowResized();
+
+        // required if executed from outside setup menu
+        SaveSetupIfNeeded();
       }
     }
 #if defined(PLATFORM_ANDROID)
@@ -1409,8 +1416,12 @@ static boolean checkTextInputKey(Key key)
   if (game_status == GAME_MODE_PLAYING)
     return FALSE;
 
-  // if Shift or right Alt key is pressed, handle key as text input
-  if ((GetKeyModState() & KMOD_TextInput) != KMOD_None)
+  // if Ctrl or Meta key is pressed, handle raw key events
+  if ((GetKeyModState() & (KMOD_Control | KMOD_Meta)) != KMOD_None)
+    return FALSE;
+
+  // if Shift or right Alt key is pressed, handle printable keys as text input
+  if ((GetKeyModState() & KMOD_TextInput) != KMOD_None && KSYM_PRINTABLE(key))
     return TRUE;
 
   // ignore raw keys as text input when not in text input mode
@@ -1505,7 +1516,8 @@ static int HandleDropFileEvent(char *filename)
   Debug("event:dropfile", "filename == '%s'", filename);
 
   // check and extract dropped zip files into correct user data directory
-  if (!strSuffixLower(filename, ".zip"))
+  if (!strSuffixLower(filename, ".zip") &&
+      !strPrefixLower(filename, "fd:"))
   {
     Warn("file '%s' not supported", filename);
 
@@ -1539,6 +1551,52 @@ static int HandleDropFileEvent(char *filename)
       directory = getUserLevelDir(NULL);
   }
 
+  // check if level or artwork set directory already exists in target directory
+  if (!CheckZipFileForDirectory(filename, directory, tree_type) &&
+      zip_file.state == ZIP_FILE_ERROR_DIR_EXISTS)
+  {
+    char message[100];
+
+    sprintf(message, "%s set already exists! Replace with new set?",
+            (tree_type == TREE_TYPE_LEVEL_DIR ? "Level" : "Artwork"));
+
+    // ask if level or artwork set directory should be replaced (and fail if not)
+    if (!Request(message, REQ_ASK))
+      return TREE_TYPE_UNDEFINED;
+
+    time_t epoch_seconds = time(NULL);
+    struct tm *now = localtime(&epoch_seconds);
+    char suffix[100];
+
+    sprintf(suffix, ".REPLACED-%04d%02d%02d-%02d%02d%02d",
+            now->tm_year + 1900,
+            now->tm_mon + 1,
+            now->tm_mday,
+            now->tm_hour,
+            now->tm_min,
+            now->tm_sec);
+
+    // workaround: rename level or artwork set directory and topmost config file
+    // (change this so that the level or artwork set directory will be deleted)
+    char *conf_file_renamed = getStringCat2(zip_file.top_dir_conf_filename, suffix);
+    char *conf_path_renamed = getStringCat2(zip_file.top_dir_path, suffix);
+    boolean success = (rename(zip_file.top_dir_conf_filename, conf_file_renamed) == 0 &&
+                       rename(zip_file.top_dir_path, conf_path_renamed) == 0);
+
+    checked_free(conf_file_renamed);
+    checked_free(conf_path_renamed);
+
+    if (!success)
+    {
+      Request("Replacing old set failed!", REQ_CONFIRM);
+
+      return TREE_TYPE_UNDEFINED;
+    }
+  }
+
+  // show "please wait" message, because extracting zip file may take some time
+  DrawScreenBeforeAddingSet(tree_type);
+
   // extract level or artwork set from zip file to target directory
   char *top_dir = ExtractZipFileIntoDirectory(filename, directory, tree_type);
 
@@ -1615,6 +1673,12 @@ static void HandleDropCompleteEvent(int num_level_sets_succeeded,
 
 void HandleDropEvent(Event *event)
 {
+  Debug("event:drop", (event->type == SDL_DROPBEGIN	? "SDL_DROPBEGIN" :
+		       event->type == SDL_DROPFILE	? "SDL_DROPFILE" :
+		       event->type == SDL_DROPTEXT	? "SDL_DROPTEXT" :
+		       event->type == SDL_DROPCOMPLETE	? "SDL_DROPCOMPLETE" :
+		       "(unknown drop event type)"));
+
   static boolean confirm_on_drop_complete = FALSE;
   static int num_level_sets_succeeded = 0;
   static int num_artwork_sets_succeeded = 0;
@@ -1810,6 +1874,10 @@ void HandleButton(int mx, int my, int button, int button_nr)
       HandleInfoScreen(mx, my, 0, 0, button);
       break;
 
+    case GAME_MODE_STORY:
+      HandleStoryScreen(mx, my, 0, 0, button);
+      break;
+
     case GAME_MODE_SETUP:
       HandleSetupScreen(mx, my, 0, 0, button);
       break;
@@ -1986,6 +2054,31 @@ static void HandleKeysSpecial(Key key)
   }
 }
 
+static boolean HandleKeysSpeed(Key key, int key_status)
+{
+  if (game_status == GAME_MODE_PLAYING)
+  {
+    if (key == setup.shortcut.speed_fast ||
+	key == setup.shortcut.speed_slow)
+    {
+      int speed_factor = 4;
+
+      GameFrameDelay = (key_status != KEY_PRESSED ? setup.game_frame_delay :
+			key == setup.shortcut.speed_fast ? setup.game_frame_delay / speed_factor :
+			key == setup.shortcut.speed_slow ? setup.game_frame_delay * speed_factor :
+			setup.game_frame_delay);
+
+      GameFrameDelay = MIN(MAX(1, GameFrameDelay), 1000);
+
+      SetVideoFrameDelay(GameFrameDelay);
+
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
 boolean HandleKeysDebug(Key key, int key_status)
 {
 #ifdef DEBUG
@@ -2038,8 +2131,8 @@ boolean HandleKeysDebug(Key key, int key_status)
     }
     else if (key == KSYM_v)
     {
-      Debug("event:key:debug", "currently using game engine version %d",
-	    game.engine_version);
+      Debug("event:key:debug", "currently using game engine version %s",
+	    getVersionString(game.engine_version));
 
       return TRUE;
     }
@@ -2070,11 +2163,41 @@ void HandleKey(Key key, int key_status)
     { &ski.snap,  NULL,            DEFAULT_KEY_SNAP,  JOY_BUTTON_SNAP },
     { &ski.drop,  NULL,            DEFAULT_KEY_DROP,  JOY_BUTTON_DROP }
   };
+  boolean game_key_pressed = FALSE;
+  boolean game_button_key_pressed = FALSE;
   int joy = 0;
   int i;
 
-  if (HandleKeysDebug(key, key_status))
-    return;		// do not handle already processed keys again
+  // check if any game key is pressed (direction/snap/drop keys)
+  if (game_status == GAME_MODE_PLAYING)
+  {
+    int pnr;
+
+    for (pnr = 0; pnr < MAX_PLAYERS; pnr++)
+    {
+      ski = setup.input[pnr].key;
+
+      for (i = 0; i < NUM_PLAYER_ACTIONS; i++)
+	if (key == *key_info[i].key_custom)
+          game_key_pressed = TRUE;
+    }
+
+    ssi = setup.shortcut;
+
+    for (i = 0; i < NUM_DIRECTIONS; i++)
+      if (key == *key_info[i].key_snap)
+        game_key_pressed = TRUE;
+  }
+
+  // only handle speed or debug keys if no game key is pressed
+  if (!game_key_pressed)
+  {
+    if (HandleKeysSpeed(key, key_status))
+      return;		// do not handle already processed keys again
+
+    if (HandleKeysDebug(key, key_status))
+      return;		// do not handle already processed keys again
+  }
 
   // map special keys (media keys / remote control buttons) to default keys
   if (key == KSYM_PlayPause)
@@ -2122,6 +2245,10 @@ void HandleKey(Key key, int key_status)
 	    tape.property_bits |= TAPE_PROPERTY_TAS_KEYS;
 	  }
 	}
+
+        // also handle suicide key for the first player only
+        if (key == setup.shortcut.suicide)
+          key_action |= KEY_SUICIDE;
       }
 
       if (key_status == KEY_PRESSED)
@@ -2187,6 +2314,18 @@ void HandleKey(Key key, int key_status)
       // for any keyboard event, enable playfield mouse cursor
       if (key_action && key_status == KEY_PRESSED)
 	SetPlayfieldMouseCursorEnabled(TRUE);
+
+      // check if any game button key was pressed (snap or drop key)
+      if (key_action & JOY_BUTTON)
+        game_button_key_pressed = TRUE;
+    }
+
+    // check if key to toggle extra game panel items was pressed or released
+    if (key == setup.shortcut.toggle_panel)
+    {
+      game.panel.show_extra_items = (key_status == KEY_PRESSED);
+
+      DrawGameDoorValues_ForceRedraw();
     }
   }
   else
@@ -2285,7 +2424,7 @@ void HandleKey(Key key, int key_status)
   }
 
   if (game_status == GAME_MODE_PLAYING && game.all_players_gone &&
-      (key == KSYM_Return || key == setup.shortcut.toggle_pause))
+      (key == KSYM_Return || key == setup.shortcut.toggle_pause || game_button_key_pressed))
   {
     GameEnd();
 
@@ -2383,6 +2522,7 @@ void HandleKey(Key key, int key_status)
     case GAME_MODE_LEVELNR:
     case GAME_MODE_SETUP:
     case GAME_MODE_INFO:
+    case GAME_MODE_STORY:
     case GAME_MODE_SCORES:
     case GAME_MODE_SCOREINFO:
 
@@ -2406,7 +2546,9 @@ void HandleKey(Key key, int key_status)
 	  else if (game_status == GAME_MODE_SETUP)
 	    HandleSetupScreen(0, 0, 0, 0, MB_MENU_CHOICE);
 	  else if (game_status == GAME_MODE_INFO)
-	    HandleInfoScreen(0, 0, 0, 0, MB_MENU_CHOICE);
+	    HandleInfoScreen(0, 0, 0, 0, (key == KSYM_space ? MB_MENU_CONTINUE : MB_MENU_CHOICE));
+	  else if (game_status == GAME_MODE_STORY)
+	    HandleStoryScreen(0, 0, 0, 0, (key == KSYM_space ? MB_MENU_CONTINUE : MB_MENU_CHOICE));
 	  else if (game_status == GAME_MODE_SCORES)
 	    HandleHallOfFame(0, 0, 0, 0, MB_MENU_CHOICE);
 	  else if (game_status == GAME_MODE_SCOREINFO)
@@ -2429,6 +2571,8 @@ void HandleKey(Key key, int key_status)
 	    HandleSetupScreen(0, 0, 0, 0, MB_MENU_LEAVE);
 	  else if (game_status == GAME_MODE_INFO)
 	    HandleInfoScreen(0, 0, 0, 0, MB_MENU_LEAVE);
+	  else if (game_status == GAME_MODE_STORY)
+	    HandleStoryScreen(0, 0, 0, 0, MB_MENU_LEAVE);
 	  else if (game_status == GAME_MODE_SCORES)
 	    HandleHallOfFame(0, 0, 0, 0, MB_MENU_LEAVE);
 	  else if (game_status == GAME_MODE_SCOREINFO)
@@ -2446,6 +2590,8 @@ void HandleKey(Key key, int key_status)
 	    HandleSetupScreen(0, 0, 0, -1 * SCROLL_PAGE, MB_MENU_MARK);
 	  else if (game_status == GAME_MODE_INFO)
 	    HandleInfoScreen(0, 0, 0, -1 * SCROLL_PAGE, MB_MENU_MARK);
+	  else if (game_status == GAME_MODE_STORY)
+	    HandleStoryScreen(0, 0, 0, -1 * SCROLL_PAGE, MB_MENU_MARK);
 	  else if (game_status == GAME_MODE_SCORES)
 	    HandleHallOfFame(0, 0, 0, -1 * SCROLL_PAGE, MB_MENU_MARK);
 	  else if (game_status == GAME_MODE_SCOREINFO)
@@ -2463,6 +2609,8 @@ void HandleKey(Key key, int key_status)
 	    HandleSetupScreen(0, 0, 0, +1 * SCROLL_PAGE, MB_MENU_MARK);
 	  else if (game_status == GAME_MODE_INFO)
 	    HandleInfoScreen(0, 0, 0, +1 * SCROLL_PAGE, MB_MENU_MARK);
+	  else if (game_status == GAME_MODE_STORY)
+	    HandleStoryScreen(0, 0, 0, +1 * SCROLL_PAGE, MB_MENU_MARK);
 	  else if (game_status == GAME_MODE_SCORES)
 	    HandleHallOfFame(0, 0, 0, +1 * SCROLL_PAGE, MB_MENU_MARK);
 	  else if (game_status == GAME_MODE_SCOREINFO)
@@ -2699,6 +2847,7 @@ void HandleJoystick(void)
     case GAME_MODE_LEVELNR:
     case GAME_MODE_SETUP:
     case GAME_MODE_INFO:
+    case GAME_MODE_STORY:
     case GAME_MODE_SCORES:
     case GAME_MODE_SCOREINFO:
     {
@@ -2719,6 +2868,8 @@ void HandleJoystick(void)
 	HandleSetupScreen(0,0,dx,dy, newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
       else if (game_status == GAME_MODE_INFO)
 	HandleInfoScreen(0,0,dx,dy, newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
+      else if (game_status == GAME_MODE_STORY)
+	HandleStoryScreen(0,0,dx,dy, newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
       else if (game_status == GAME_MODE_SCORES)
 	HandleHallOfFame(0,0,dx,dy, newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
       else if (game_status == GAME_MODE_SCOREINFO)
