@@ -22,6 +22,8 @@
 #define STD_NUM_KEYS			4
 #define MAX_NUM_KEYS			8
 
+#define NUM_BDX_KEYS			3
+
 #define NUM_BELTS			4
 #define NUM_BELT_PARTS			3
 
@@ -52,6 +54,9 @@ struct GamePanelInfo
 {
   struct TextPosInfo level_number;
   struct TextPosInfo gems;
+  struct TextPosInfo gems_needed;
+  struct TextPosInfo gems_collected;
+  struct TextPosInfo gems_score;
   struct TextPosInfo inventory_count;
   struct TextPosInfo inventory_first[NUM_PANEL_INVENTORY];
   struct TextPosInfo inventory_last[NUM_PANEL_INVENTORY];
@@ -102,12 +107,41 @@ struct GamePanelInfo
   struct TextPosInfo element_count[NUM_PANEL_ELEMENTS];
   struct TextPosInfo ce_score[NUM_PANEL_CE_SCORE];
   struct TextPosInfo ce_score_element[NUM_PANEL_CE_SCORE];
+  struct TextPosInfo bdx_lives;
+  struct TextPosInfo bdx_key[NUM_BDX_KEYS];
+  struct TextPosInfo bdx_key_count[NUM_BDX_KEYS];
+  struct TextPosInfo bdx_diamond_key;
+  struct TextPosInfo bdx_gravity;
+  struct TextPosInfo bdx_gravity_next;
+  struct TextPosInfo bdx_gravity_time;
+  struct TextPosInfo bdx_gravity_state;
+  struct TextPosInfo bdx_skeleton;
+  struct TextPosInfo bdx_skeleton_count;
+  struct TextPosInfo bdx_sweet;
+  struct TextPosInfo bdx_pneumatic_hammer;
+  struct TextPosInfo bdx_rocket_count;
+  struct TextPosInfo bdx_rocket_state;
+  struct TextPosInfo bdx_magic_wall;
+  struct TextPosInfo bdx_magic_wall_time;
+  struct TextPosInfo bdx_creature_switch;
+  struct TextPosInfo bdx_expandable_wall_switch;
+  struct TextPosInfo bdx_biter_switch_time;
+  struct TextPosInfo bdx_replicator;
+  struct TextPosInfo bdx_replicator_switch;
+  struct TextPosInfo bdx_conveyor_left;
+  struct TextPosInfo bdx_conveyor_right;
+  struct TextPosInfo bdx_conveyor_switch;
+  struct TextPosInfo bdx_conveyor_dir_switch;
   struct TextPosInfo player_name;
+  struct TextPosInfo levelset_name;
   struct TextPosInfo level_name;
   struct TextPosInfo level_author;
 
   // value to determine if panel will be updated or not
   boolean active;
+
+  // value for dynamically showing extra panel items
+  boolean show_extra_items;
 };
 
 struct GameButtonInfo
@@ -163,19 +197,24 @@ struct GameInfo
 
   // values for graphics engine customization
   int graphics_engine_version;
+  boolean use_native_bd_graphics_engine;
   boolean use_native_emc_graphics_engine;
   boolean use_native_sp_graphics_engine;
   boolean use_masked_pushing;
   boolean use_masked_elements;
   boolean use_masked_elements_initial;
+  boolean keep_panel_open_when_restarting;
   int forced_scroll_x;
   int forced_scroll_y;
   int forced_scroll_delay_value;
   int scroll_delay_value;
   int tile_size;
 
+  // values for sound engine customization
+  boolean use_native_bd_sound_engine;
+
   // constant within running game
-  int engine_version;
+  VersionType engine_version;
   int emulation;
   int initial_move_delay[MAX_PLAYERS];
   int initial_move_delay_value[MAX_PLAYERS];
@@ -209,7 +248,6 @@ struct GameInfo
   int wind_direction;
 
   boolean explosions_delayed;
-  boolean envelope_active;
   boolean no_level_time_limit;	// (variable only in very special case)
   boolean time_limit;		// forced by levelset config or setup option
 
@@ -250,7 +288,10 @@ struct GameInfo
   boolean restart_level;
 
   // values for special request dialog control
+  boolean request_open;
   boolean request_active;
+  boolean envelope_active;
+  boolean any_door_active;
 
   // values for special game control
   int centered_player_nr;
@@ -282,6 +323,9 @@ struct GameInfo
   int LevelSolved_CountingHealth;
 
   boolean RestartGameRequested;
+  boolean InitGameRequested;
+
+  boolean InitColorTemplateImagesNeeded;
 };
 
 struct PlayerInfo
@@ -419,6 +463,7 @@ int getPlayerInventorySize(int);
 
 void UpdateGameDoorValues(void);
 void DrawGameDoorValues(void);
+void DrawGameDoorValues_ForceRedraw(void);
 
 void UpdateAndDisplayGameControlValues(void);
 
@@ -426,7 +471,6 @@ void InitGameSound(void);
 void InitGame(void);
 
 void UpdateEngineValues(int, int, int, int);
-void GameWon(void);
 void GameEnd(void);
 
 void MergeServerScore(void);
@@ -441,6 +485,7 @@ void DrawDynamite(int, int);
 void StartGameActions(boolean, boolean, int);
 
 void GameActions(void);
+void GameActions_BD_Main(void);
 void GameActions_EM_Main(void);
 void GameActions_SP_Main(void);
 void GameActions_MM_Main(void);
@@ -464,9 +509,12 @@ void RequestQuitGameExt(boolean, boolean, char *);
 void RequestQuitGame(boolean);
 
 boolean CheckRestartGame(void);
+boolean checkGameRunning(void);
+boolean checkGamePlaying(void);
 boolean checkGameSolved(void);
 boolean checkGameFailed(void);
 boolean checkGameEnded(void);
+boolean checkRequestActive(void);
 
 unsigned int InitEngineRandom_RND(int);
 unsigned int RND(int);
