@@ -1,3 +1,14 @@
+// ============================================================================
+// Rocks'n'Diamonds - McDuffin Strikes Back!
+// ----------------------------------------------------------------------------
+// (c) 1995-2024 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    https://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// main_mm.h
+// ============================================================================
+
 #ifndef MAIN_MM_H
 #define MAIN_MM_H
 
@@ -5,15 +16,14 @@
 // external functions and definitions imported from main program to game_mm
 // ============================================================================
 
-#include "../engines.h"
-#include "../conf_gfx.h"
+#include "import_mm.h"
 
 
 // ============================================================================
 // functions and definitions that are exported from game_mm to main program
 // ============================================================================
 
-#include "export.h"
+#include "export_mm.h"
 
 
 // ============================================================================
