@@ -165,35 +165,35 @@ void DrawGraphicShifted_MM(int x, int y, int dx, int dy, int graphic,
     return;
   }
 
-  if (dx || dy)			// Verschiebung der Grafik?
+  if (dx || dy)				// Verschiebung der Grafik?
   {
-    if (x < BX1)		// Element kommt von links ins Bild
+    if (x < BX1)			// Element kommt von links ins Bild
     {
       x = BX1;
       width = dx;
       cx = TILEX - dx;
       dx = 0;
     }
-    else if (x > BX2)		// Element kommt von rechts ins Bild
+    else if (x > BX2)			// Element kommt von rechts ins Bild
     {
       x = BX2;
       width = -dx;
       dx = TILEX + dx;
     }
-    else if (x==BX1 && dx < 0)	// Element verläßt links das Bild
+    else if (x == BX1 && dx < 0)	// Element verläßt links das Bild
     {
       width += dx;
       cx = -dx;
       dx = 0;
     }
-    else if (x==BX2 && dx > 0)	// Element verläßt rechts das Bild
+    else if (x == BX2 && dx > 0)	// Element verläßt rechts das Bild
       width -= dx;
-    else if (dx)		// allg. Bewegung in x-Richtung
+    else if (dx)			// allg. Bewegung in x-Richtung
       MarkTileDirty(x + SIGN(dx), y);
 
-    if (y < BY1)		// Element kommt von oben ins Bild
+    if (y < BY1)			// Element kommt von oben ins Bild
     {
-      if (cut_mode==CUT_BELOW)	// Element oberhalb des Bildes
+      if (cut_mode == CUT_BELOW)	// Element oberhalb des Bildes
 	return;
 
       y = BY1;
@@ -201,13 +201,13 @@ void DrawGraphicShifted_MM(int x, int y, int dx, int dy, int graphic,
       cy = TILEY - dy;
       dy = 0;
     }
-    else if (y > BY2)		// Element kommt von unten ins Bild
+    else if (y > BY2)			// Element kommt von unten ins Bild
     {
       y = BY2;
       height = -dy;
       dy = TILEY + dy;
     }
-    else if (y==BY1 && dy < 0)	// Element verläßt oben das Bild
+    else if (y == BY1 && dy < 0)	// Element verläßt oben das Bild
     {
       height += dy;
       cy = -dy;
@@ -215,19 +215,19 @@ void DrawGraphicShifted_MM(int x, int y, int dx, int dy, int graphic,
     }
     else if (dy > 0 && cut_mode == CUT_ABOVE)
     {
-      if (y == BY2)		// Element unterhalb des Bildes
+      if (y == BY2)			// Element unterhalb des Bildes
 	return;
 
       height = dy;
       cy = TILEY - dy;
       dy = TILEY;
       MarkTileDirty(x, y + 1);
-    }				// Element verläßt unten das Bild
+    }					// Element verläßt unten das Bild
     else if (dy > 0 && (y == BY2 || cut_mode == CUT_BELOW))
     {
       height -= dy;
     }
-    else if (dy)		// allg. Bewegung in y-Richtung
+    else if (dy)			// allg. Bewegung in y-Richtung
     {
       MarkTileDirty(x, y + SIGN(dy));
     }
@@ -845,14 +845,14 @@ static void DrawTileCursor_Xsn(int draw_target)
     if (debug)
       setup.debug.xsn_percent = percent;
 
-    if (setup.debug.xsn_mode != AUTO)
+    if (setup.debug.xsn_mode != STATE_AUTO)
       percent = setup.debug.xsn_percent;
 
     setup.debug.xsn_percent = percent;
 
     active = (percent > 0);
 
-    if ((active && !active_last) || setup.debug.xsn_mode != AUTO)
+    if ((active && !active_last) || setup.debug.xsn_mode != STATE_AUTO)
       removeHideSetupEntry(&setup.debug.xsn_mode);
     else if (!active && active_last)
       setHideSetupEntry(&setup.debug.xsn_mode);
@@ -939,9 +939,6 @@ static void DrawTileCursor_Xsn(int draw_target)
     xsn.change_dir   = 0;
 
     xsn.alpha = XSN_ALPHA_DEFAULT;
-
-    for (i = 0; i < xsn.max_items; i++)
-      xsn_init_item(i);
   }
 
   if (xsn.area_xsize != gfx.win_xsize ||
@@ -951,6 +948,9 @@ static void DrawTileCursor_Xsn(int draw_target)
     xsn.area_xsize = gfx.win_xsize;
     xsn.area_ysize = gfx.win_ysize;
 
+    for (i = 0; i < xsn.max_items; i++)
+      xsn_init_item(i);
+
     if (xsn.bitmap != NULL)
       FreeBitmap(xsn.bitmap);
 
@@ -1057,6 +1057,16 @@ static void DrawTileCursor_Xsn(int draw_target)
   }
 }
 
+static void FreeTileCursorTextures_Xsn(void)
+{
+  int i;
+
+  SDLFreeBitmapTextures(xsn.bitmap);
+
+  for (i = 0; i < num_xsn_data; i++)
+    SDLFreeBitmapTextures(xsn_data[i].bitmap);
+}
+
 void DrawTileCursor_MM(int draw_target, int drawing_stage,
 		       boolean tile_cursor_active)
 {
@@ -1124,6 +1134,11 @@ void DrawTileCursor_MM(int draw_target, int drawing_stage,
 		     dst_x, dst_y);
 }
 
+void FreeTileCursorTextures_MM(void)
+{
+  FreeTileCursorTextures_Xsn();
+}
+
 Pixel ReadPixel(DrawBuffer *bitmap, int x, int y)
 {
   return GetPixel(bitmap, x, y);
