@@ -11,6 +11,7 @@
 
 #include <stdarg.h>
 #include <string.h>
+#include <math.h>
 
 #include "gadgets.h"
 #include "image.h"
@@ -45,6 +46,466 @@ static void (*PlayGadgetSoundActivating)(void) = NULL;
 static void (*PlayGadgetSoundSelecting)(void) = NULL;
 
 
+// Color picker by Brooke Tilley (pixlark) from https://github.com/pixlark/ColorPicker
+// RGB/HSV conversions by David H from https://stackoverflow.com/questions/3018313/
+
+// A bunch of magic numbers for UI positioning (used by DrawColorPicker() and its functions)
+#define CP_SEPARATOR_SIZE	(1)
+#define CP_GADGET_WIDTH		(256)
+#define CP_GADGET_HEIGHT	(300 + CP_SEPARATOR_SIZE)
+#define CP_MAIN_GRADIENT_X	(0)
+#define CP_MAIN_GRADIENT_Y	(0)
+#define CP_MAIN_GRADIENT_WIDTH	(CP_GADGET_WIDTH)
+#define CP_MAIN_GRADIENT_HEIGHT	(CP_MAIN_GRADIENT_WIDTH)
+#define CP_SAMPLE_BOX_X		(CP_HUE_GRADIENT_WIDTH + CP_SEPARATOR_SIZE)
+#define CP_SAMPLE_BOX_Y		(CP_MAIN_GRADIENT_HEIGHT + CP_SEPARATOR_SIZE)
+#define CP_SAMPLE_BOX_WIDTH	(CP_SAMPLE_BOX_HEIGHT)
+#define CP_SAMPLE_BOX_HEIGHT	(CP_GADGET_HEIGHT - CP_MAIN_GRADIENT_HEIGHT - CP_SEPARATOR_SIZE)
+#define CP_HUE_GRADIENT_X	(0)
+#define CP_HUE_GRADIENT_Y	(CP_SAMPLE_BOX_Y)
+#define CP_HUE_GRADIENT_WIDTH	(CP_GADGET_WIDTH - CP_SAMPLE_BOX_WIDTH - CP_SEPARATOR_SIZE)
+#define CP_HUE_GRADIENT_HEIGHT	(CP_SAMPLE_BOX_HEIGHT / 2)
+#define CP_COLOR_TEXT_X		(0)
+#define CP_COLOR_TEXT_Y		(CP_HUE_GRADIENT_Y + CP_HUE_GRADIENT_HEIGHT + CP_SEPARATOR_SIZE)
+#define CP_MARK_SIZE		(3)
+
+#define CP_INSIDE_MAIN(x, y)	((x) >= CP_MAIN_GRADIENT_X &&				\
+                                 (x) <  CP_MAIN_GRADIENT_X + CP_MAIN_GRADIENT_WIDTH &&	\
+                                 (y) >= CP_MAIN_GRADIENT_Y &&				\
+                                 (y) <  CP_MAIN_GRADIENT_Y + CP_MAIN_GRADIENT_HEIGHT)
+#define CP_INSIDE_HUE(x, y)	((x) >= CP_HUE_GRADIENT_X &&				\
+                                 (x) <  CP_HUE_GRADIENT_X + CP_HUE_GRADIENT_WIDTH &&	\
+                                 (y) >= CP_HUE_GRADIENT_Y &&				\
+                                 (y) <  CP_HUE_GRADIENT_Y + CP_HUE_GRADIENT_HEIGHT)
+#define CP_INSIDE_SAMPLE(x, y)	((x) >= CP_SAMPLE_BOX_X &&				\
+                                 (x) <  CP_SAMPLE_BOX_X + CP_SAMPLE_BOX_WIDTH &&	\
+                                 (y) >= CP_SAMPLE_BOX_Y &&				\
+                                 (y) <  CP_SAMPLE_BOX_Y + CP_SAMPLE_BOX_HEIGHT)
+
+// Start of David H's conversion code
+static HSVColor get_hsv_from_rgb(RGBColor in)
+{
+  HSVColor out;
+  double min, max, delta;
+
+  min = in.r < in.g ? in.r : in.g;
+  min = min  < in.b ? min  : in.b;
+
+  max = in.r > in.g ? in.r : in.g;
+  max = max  > in.b ? max  : in.b;
+
+  out.v = max;                  // v
+  delta = max - min;
+
+  if (delta < 0.00001)
+  {
+    out.s = 0;
+    out.h = 0; // undefined, maybe nan?
+
+    return out;
+  }
+
+  if (max > 0.0)
+  {
+    // NOTE: if Max is == 0, this divide would cause a crash
+    out.s = (delta / max);      // s
+  }
+  else
+  {
+    // if max is 0, then r = g = b = 0
+    // s = 0, h is undefined
+    out.s = 0.0;
+    out.h = NAN;                            // its now undefined
+
+    return out;
+  }
+
+  if (in.r >= max)                          // > is bogus, just keeps compilor happy
+    out.h = (in.g - in.b) / delta;          // between yellow & magenta
+  else if (in.g >= max)
+    out.h = 2.0 + (in.b - in.r) / delta;    // between cyan & yellow
+  else
+    out.h = 4.0 + (in.r - in.g) / delta;    // between magenta & cyan
+
+  out.h *= 60.0;                            // degrees
+
+  if (out.h < 0.0)
+    out.h += 360.0;
+
+  return out;
+}
+
+static RGBColor get_rgb_from_hsv(HSVColor in)
+{
+  double hh, p, q, t, ff;
+  RGBColor out;
+  int i;
+
+  if (in.s <= 0.0)
+  {
+    // < is bogus, just shuts up warnings
+    out.r = in.v;
+    out.g = in.v;
+    out.b = in.v;
+
+    return out;
+  }
+
+  hh = in.h;
+  if (hh >= 360.0)
+    hh = 0.0;
+
+  hh /= 60.0;
+  i = (int)hh;
+  ff = hh - i;
+  p = in.v * (1.0 - in.s);
+  q = in.v * (1.0 - (in.s * ff));
+  t = in.v * (1.0 - (in.s * (1.0 - ff)));
+
+  switch (i)
+  {
+    case 0:
+      out.r = in.v;
+      out.g = t;
+      out.b = p;
+      break;
+
+    case 1:
+      out.r = q;
+      out.g = in.v;
+      out.b = p;
+      break;
+
+    case 2:
+      out.r = p;
+      out.g = in.v;
+      out.b = t;
+      break;
+
+    case 3:
+      out.r = p;
+      out.g = q;
+      out.b = in.v;
+      break;
+
+    case 4:
+      out.r = t;
+      out.g = p;
+      out.b = in.v;
+      break;
+
+    case 5:
+    default:
+      out.r = in.v;
+      out.g = p;
+      out.b = q;
+      break;
+  }
+
+  return out;
+}
+// End of David H's conversion code
+
+enum
+{
+  CP_STATE_NONE,
+  CP_STATE_MAIN_GRADIENT_CHANGE,
+  CP_STATE_HUE_GRADIENT_CHANGE,
+  CP_STATE_SAMPLE_BOX,
+};
+
+static RGBColor get_rgb_from_int(int color_int)
+{
+  RGBColor color_rgb =
+  {
+    ((color_int >> 16) & 255) / 255.0,
+    ((color_int >>  8) & 255) / 255.0,
+    ((color_int      ) & 255) / 255.0,
+  };
+
+  return color_rgb;
+}
+
+static int get_int_from_rgb(RGBColor color_rgb)
+{
+  int color_int = ((int)(color_rgb.r * 255) << 16 |
+                   (int)(color_rgb.g * 255) << 8  |
+                   (int)(color_rgb.b * 255));
+
+  return color_int;
+}
+
+static SDL_Color get_sdl_from_rgb(RGBColor color_rgb)
+{
+  SDL_Color color_sdl =
+  {
+    color_rgb.r * 255,
+    color_rgb.g * 255,
+    color_rgb.b * 255,
+    0xff
+  };
+
+  return color_sdl;
+}
+
+static int get_int_from_hsv(HSVColor color_hsv)
+{
+  return get_int_from_rgb(get_rgb_from_hsv(color_hsv));
+}
+
+static HSVColor get_hsv_from_int(int color_int)
+{
+  return get_hsv_from_rgb(get_rgb_from_int(color_int));
+}
+
+static SDL_Color get_sdl_from_hsv(HSVColor color_hsv)
+{
+  return get_sdl_from_rgb(get_rgb_from_hsv(color_hsv));
+}
+
+static SDL_Color get_sdl_from_int(int color_int)
+{
+  return get_sdl_from_rgb(get_rgb_from_int(color_int));
+}
+
+static void set_pixel(SDL_Surface *surface, SDL_Color color, int x, int y)
+{
+  ((char *)surface->pixels)[(x + y * surface->w) * 4    ] = color.r;
+  ((char *)surface->pixels)[(x + y * surface->w) * 4 + 1] = color.g;
+  ((char *)surface->pixels)[(x + y * surface->w) * 4 + 2] = color.b;
+  ((char *)surface->pixels)[(x + y * surface->w) * 4 + 3] = color.a;
+}
+
+static void DrawColorPicker_Gradient(SDL_Surface *surface, double hue)
+{
+  int xpos = CP_MAIN_GRADIENT_X;
+  int ypos = CP_MAIN_GRADIENT_Y;
+  int xsize = CP_MAIN_GRADIENT_WIDTH;
+  int ysize = CP_MAIN_GRADIENT_HEIGHT;
+  int x, y;
+
+  for (y = 0; y < ysize; y++)
+  {
+    for (x = 0; x < xsize; x++)
+    {
+      HSVColor color_hsv =
+      {
+        hue,
+        (double) x / xsize,
+        1.0 - ((double) y / ysize),
+      };
+
+      set_pixel(surface, get_sdl_from_hsv(color_hsv), xpos + x, ypos + y);
+    }
+  }
+}
+
+static void DrawColorPicker_HueGradient(SDL_Surface *surface)
+{
+  int xpos = CP_HUE_GRADIENT_X;
+  int ypos = CP_HUE_GRADIENT_Y;
+  int xsize = CP_HUE_GRADIENT_WIDTH;
+  int ysize = CP_HUE_GRADIENT_HEIGHT;
+  int x, y;
+
+  for (y = 0; y < ysize; y++)
+  {
+    for (x = 0; x < xsize; x++)
+    {
+      HSVColor color_hsv =
+      {
+        (double) x / xsize * 360,
+        1.0,
+        1.0,
+      };
+
+      set_pixel(surface, get_sdl_from_hsv(color_hsv), xpos + x, ypos + y);
+    }
+  }
+}
+
+static void DrawColorPicker_Table(SDL_Surface *surface, struct GadgetInfo *gi)
+{
+  int *values = gi->colorpicker.values;
+  int count = gi->colorpicker.count;
+  int table_size = sqrt(count);
+  int box_size = CP_GADGET_WIDTH / table_size;
+  int x, y;
+
+  for (y = 0; y < table_size; y++)
+  {
+    for (x = 0; x < table_size; x++)
+    {
+      SDL_Rect draw_rect =
+      {
+        x * box_size,
+        y * box_size,
+        box_size,
+        box_size,
+      };
+      int pos = y * table_size + x;
+      int value = values[pos];
+      SDL_Color color = get_sdl_from_int(value);
+      Pixel pixel = SDL_MapRGB(surface->format, color.r, color.g, color.b);
+
+      SDL_FillRect(surface, &draw_rect, pixel);
+    }
+  }
+}
+
+static void DrawColorPicker_ColorMarker(struct GadgetInfo *gi)
+{
+  if (gi->colorpicker.count == 0)	// RGB colors
+  {
+    int x = gi->x + gi->border.xsize;
+    int y = gi->y + gi->border.ysize;
+    int xsize_main = CP_MAIN_GRADIENT_WIDTH;
+    int ysize_main = CP_MAIN_GRADIENT_HEIGHT;
+    int xsize_hue  = CP_HUE_GRADIENT_WIDTH;
+    int ysize_hue  = CP_HUE_GRADIENT_HEIGHT;
+    int x_main = x + xsize_main * gi->colorpicker.color_hsv.s;
+    int y_main = y + ysize_main * (1.0 - gi->colorpicker.color_hsv.v);
+    int x_hue = x + (((gi->colorpicker.color_hsv.h + 1) / 360.0) * xsize_hue);
+    int y_hue = y + CP_HUE_GRADIENT_Y;
+
+    // draw color position indicator
+    FillRectangle(drawto, x_main - CP_MARK_SIZE / 2, y, CP_MARK_SIZE, ysize_main, BLACK_PIXEL);
+    FillRectangle(drawto, x, y_main - CP_MARK_SIZE / 2, xsize_main, CP_MARK_SIZE, BLACK_PIXEL);
+    FillRectangle(drawto, x_main, y, 1, ysize_main, WHITE_PIXEL);
+    FillRectangle(drawto, x, y_main, xsize_main, 1, WHITE_PIXEL);
+
+    // draw color slider
+    FillRectangle(drawto, x_hue - CP_MARK_SIZE / 2, y_hue, CP_MARK_SIZE, ysize_hue, BLACK_PIXEL);
+    FillRectangle(drawto, x_hue, y_hue, 1, ysize_hue, WHITE_PIXEL);
+  }
+  else					// indexed colors
+  {
+    int count = gi->colorpicker.count;
+    int table_size = sqrt(count);
+    int box_size = CP_GADGET_WIDTH / table_size;
+    int pos = gi->colorpicker.value;
+    int xpos = pos % table_size;
+    int ypos = pos / table_size;
+    int x = gi->x + gi->border.xsize + xpos * box_size;
+    int y = gi->y + gi->border.ysize + ypos * box_size;
+
+    // draw color position indicator
+    FillRectangle(drawto, x - 1, y - 1, box_size + 2, CP_MARK_SIZE, BLACK_PIXEL);
+    FillRectangle(drawto, x + box_size - 2, y - 1, CP_MARK_SIZE, box_size + 2, BLACK_PIXEL);
+    FillRectangle(drawto, x - 1, y + box_size - 2, box_size + 2, CP_MARK_SIZE, BLACK_PIXEL);
+    FillRectangle(drawto, x - 1, y - 1, CP_MARK_SIZE, box_size + 2, BLACK_PIXEL);
+    FillRectangle(drawto, x, y, box_size, 1, WHITE_PIXEL);
+    FillRectangle(drawto, x + box_size - 1, y, 1, box_size, WHITE_PIXEL);
+    FillRectangle(drawto, x, y + box_size - 1, box_size, 1, WHITE_PIXEL);
+    FillRectangle(drawto, x, y, 1, box_size, WHITE_PIXEL);
+  }
+}
+
+static void DrawColorPicker_SampleBox(SDL_Surface *surface, struct GadgetInfo *gi)
+{
+  SDL_Rect draw_rect =
+  {
+    CP_SAMPLE_BOX_X,
+    CP_SAMPLE_BOX_Y,
+    CP_SAMPLE_BOX_WIDTH,
+    CP_SAMPLE_BOX_HEIGHT,
+  };
+
+  RGBColor color_rgb;
+
+  if (gi->colorpicker.count == 0)	// RGB colors
+  {
+    color_rgb = get_rgb_from_hsv(gi->colorpicker.color_hsv);
+  }
+  else					// indexed colors
+  {
+    int *values = gi->colorpicker.values;
+    int value = values[gi->colorpicker.value];
+
+    color_rgb = get_rgb_from_int(value);
+  }
+
+  SDL_Color color = get_sdl_from_rgb(color_rgb);
+  Pixel pixel = SDL_MapRGB(surface->format, color.r, color.g, color.b);
+
+  SDL_FillRect(surface, &draw_rect, pixel);
+}
+
+static void DrawColorPicker_ColorText(struct GadgetInfo *gi)
+{
+  int x = gi->x + gi->border.xsize;
+  int y = gi->y + gi->border.ysize;;
+  int font_nr = gi->font;
+  int font_height = getFontHeight(font_nr);
+  int font_padding = (CP_HUE_GRADIENT_HEIGHT - font_height) / 2;
+  int x_text = x + CP_COLOR_TEXT_X + font_padding;
+  int y_text = y + CP_COLOR_TEXT_Y + font_padding;
+
+  int draw_background_mask = GetDrawBackgroundMask();
+
+  SetDrawBackgroundMask(REDRAW_NONE);	// deactivate masked drawing (use black background)
+
+  if (gi->colorpicker.count == 0)
+  {
+    RGBColor color_rgb = get_rgb_from_hsv(gi->colorpicker.color_hsv);
+    char text[128];
+
+    sprintf(text, "#%02x%02x%02x",
+            (int) (color_rgb.r * 255),
+            (int) (color_rgb.g * 255),
+            (int) (color_rgb.b * 255));
+
+    DrawText(x_text, y_text, text, font_nr);
+  }
+  else
+  {
+    DrawText(x_text, y_text, gi->colorpicker.names[gi->colorpicker.value], font_nr);
+  }
+
+  SetDrawBackgroundMask(draw_background_mask);
+}
+
+static void DrawColorPicker(struct GadgetInfo *gi)
+{
+  SDL_Surface *surface;
+  int x = gi->x + gi->border.xsize;
+  int y = gi->y + gi->border.ysize;;
+  int xsize = CP_GADGET_WIDTH;
+  int ysize = CP_GADGET_HEIGHT;
+
+  surface = SDL_CreateRGBSurface(0, xsize, ysize, 32, 0x0000ff, 0x00ff00, 0xff0000, 0);
+
+  if (gi->colorpicker.count == 0)
+  {
+    DrawColorPicker_Gradient(surface, gi->colorpicker.color_hsv.h);
+    DrawColorPicker_HueGradient(surface);
+  }
+  else
+  {
+    DrawColorPicker_Table(surface, gi);
+  }
+
+  DrawColorPicker_SampleBox(surface, gi);
+
+  SDLBlitSurface(surface, drawto->surface, 0, 0, xsize, ysize, x, y);
+  SDL_FreeSurface(surface);
+
+  // draw color marker
+  DrawColorPicker_ColorMarker(gi);
+
+  // draw color RGB value
+  DrawColorPicker_ColorText(gi);
+}
+
+static void SelectColorPickerColor(struct GadgetInfo *gi)
+{
+  if (gi->colorpicker.count == 0)		// RGB colors
+  {
+    gi->colorpicker.value = get_int_from_hsv(gi->colorpicker.color_hsv);
+    gi->event.type = GD_EVENT_COLOR_PICKER_LEAVING;
+  }
+}
+
 void InitGadgetsSoundCallback(void (*activating_function)(void),
 			      void (*selecting_function)(void))
 {
@@ -166,6 +627,16 @@ static struct GadgetInfo *getGadgetInfoFromMousePosition(int mx, int my,
       return gi;
   }
 
+  // color pickers may overlap other active gadgets, so check them first
+  for (gi = gadget_list_first_entry; gi != NULL; gi = gi->next)
+  {
+    if (gi->mapped && gi->active &&
+	gi->type & GD_TYPE_COLOR_PICKER &&
+	mx >= gi->x && mx < gi->x + gi->width &&
+	my >= gi->y && my < gi->y + gi->height)
+      return gi;
+  }
+
   // check all other gadgets
   for (gi = gadget_list_first_entry; gi != NULL; gi = gi->next)
   {
@@ -346,8 +817,12 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 	char text[MAX_GADGET_TEXTSIZE + 1];
 	int font_nr = (pressed ? gi->font_active : gi->font);
 	int font_width = getFontWidth(font_nr);
+	int font_height = getFontHeight(font_nr);
+	struct FontBitmapInfo *font = getFontBitmapInfo(font_nr);
 	int border_x = gi->border.xsize;
 	int border_y = gi->border.ysize;
+	int text_x = gi->x + font->draw_xoffset;
+	int text_y = gi->y + font->draw_yoffset;
 
 	// left part of gadget
 	BlitBitmapOnBackground(gd->bitmap, drawto, gd->x, gd->y,
@@ -369,6 +844,15 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 	strcpy(text, gi->textinput.value);
 	strcat(text, " ");
 
+	// dirty workaround to erase text if input gadget font has draw offset
+	if (font->draw_xoffset != 0 || font->draw_yoffset != 0)
+	  for (i = 0; i < gi->textinput.size + 1; i++)
+	    BlitBitmapOnBackground(gd->bitmap, drawto,
+				   gd->x + border_x, gd->y + border_y,
+				   font_width, font_height,
+				   text_x + border_x + i * font_width,
+				   text_y + border_y);
+
 	// gadget text value
 	DrawTextExt(drawto,
 		    gi->x + border_x, gi->y + border_y, text,
@@ -505,7 +989,7 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 
 	// gadget text value
 	DrawTextArea(x + border_x, y + border_y, gi->textarea.value,
-		     font_nr, xsize, -1, ysize, 0,
+		     font_nr, xsize, -1, ysize, -1, -1, -1, 0,
 		     BLIT_ON_BACKGROUND, FALSE, FALSE, FALSE);
 
 	cursor_letter = gi->textarea.value[gi->textarea.cursor_position];
@@ -621,13 +1105,13 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
 	    BlitBitmapOnBackground(gd->bitmap, drawto, gd->x, gd->y + border_y,
 				   border_x, font_height,
 				   gi->selectbox.x,
-				   gi->selectbox.y + border_y + i*font_height);
+				   gi->selectbox.y + border_y + i * font_height);
 	    BlitBitmapOnBackground(gd->bitmap, drawto,
 				   gd->x + gi->border.width - border_x,
 				   gd->y + border_y,
 				   border_x, font_height,
 				   gi->selectbox.x + box_width - border_x,
-				   gi->selectbox.y + border_y + i*font_height);
+				   gi->selectbox.y + border_y + i * font_height);
 	  }
 
 	  // bottom left part of gadget border
@@ -815,6 +1299,99 @@ static void DrawGadget(struct GadgetInfo *gi, boolean pressed, boolean direct)
       }
       break;
 
+    case GD_TYPE_COLOR_PICKER:
+      {
+	int i;
+	int border_x = gi->border.xsize;
+	int border_y = gi->border.ysize;
+	int x = gi->x;
+	int y = gi->y;
+	int width  = gi->width;
+	int height = gi->height;
+	int xsize = (width  - 2 * border_x) / border_x + 1;
+	int ysize = (height - 2 * border_y) / border_y + 1;
+
+	if (pressed)
+	{
+          if (!gi->colorpicker.open)
+          {
+	    gi->colorpicker.open = TRUE;
+
+            // save background under color picker
+            BlitBitmap(drawto, gfx.field_save_buffer, x, y, width, height, x, y);
+          }
+
+          // top left part of gadget border
+          BlitBitmapOnBackground(gd->bitmap, drawto, gd->x, gd->y,
+                                 border_x, border_y, x, y);
+
+          // top middle part of gadget border
+          for (i = 0; i < xsize; i++)
+            BlitBitmapOnBackground(gd->bitmap, drawto, gd->x + border_x, gd->y,
+                                   border_x, border_y,
+                                   x + border_x + i * border_x, y);
+
+          // top right part of gadget border
+          BlitBitmapOnBackground(gd->bitmap, drawto,
+                                 gd->x + gi->border.width - border_x, gd->y,
+                                 border_x, border_y,
+                                 x + width - border_x, y);
+
+          // left and right part of gadget border for each row
+          for (i = 0; i < ysize; i++)
+          {
+            BlitBitmapOnBackground(gd->bitmap, drawto, gd->x, gd->y + border_y,
+                                   border_x, border_y,
+                                   x, y + border_y + i * border_y);
+            BlitBitmapOnBackground(gd->bitmap, drawto,
+                                   gd->x + gi->border.width - border_x,
+                                   gd->y + border_y,
+                                   border_x, border_y,
+                                   x + width - border_x,
+                                   y + border_y + i * border_y);
+          }
+
+          // bottom left part of gadget border
+          BlitBitmapOnBackground(gd->bitmap, drawto,
+                                 gd->x, gd->y + gi->border.height - border_y,
+                                 border_x, border_y,
+                                 x, y + height - border_y);
+
+          // bottom middle part of gadget border
+          for (i = 0; i < xsize; i++)
+            BlitBitmapOnBackground(gd->bitmap, drawto,
+                                   gd->x + border_x,
+                                   gd->y + gi->border.height - border_y,
+                                   border_x, border_y,
+                                   x + border_x + i * border_x,
+                                   y + height - border_y);
+
+          // bottom right part of gadget border
+          BlitBitmapOnBackground(gd->bitmap, drawto,
+                                 gd->x + gi->border.width - border_x,
+                                 gd->y + gi->border.height - border_y,
+                                 border_x, border_y,
+                                 x + width - border_x,
+                                 y + height - border_y);
+
+          ClearRectangleOnBackground(drawto,
+                                     x + border_x,
+                                     y + border_y,
+                                     width - 2 * border_x,
+                                     height - 2 * border_y);
+
+          DrawColorPicker(gi);
+        }
+	else if (gi->colorpicker.open)
+	{
+	  gi->colorpicker.open = FALSE;
+
+	  // restore background under color picker
+          BlitBitmap(gfx.field_save_buffer, drawto, x, y, width, height, x, y);
+        }
+      }
+      break;
+
     default:
       return;
   }
@@ -1154,7 +1731,7 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	break;
 
       case GDI_ALT_DESIGN_UNPRESSED:
-	gi->alt_design[GD_BUTTON_UNPRESSED].bitmap= va_arg(ap, Bitmap *);
+	gi->alt_design[GD_BUTTON_UNPRESSED].bitmap = va_arg(ap, Bitmap *);
 	gi->alt_design[GD_BUTTON_UNPRESSED].x = va_arg(ap, int);
 	gi->alt_design[GD_BUTTON_UNPRESSED].y = va_arg(ap, int);
 	break;
@@ -1178,6 +1755,10 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->border.width = va_arg(ap, int);
 	break;
 
+      case GDI_DESIGN_HEIGHT:
+	gi->border.height = va_arg(ap, int);
+	break;
+
       case GDI_DECORATION_DESIGN:
 	gi->deco.design.bitmap = va_arg(ap, Bitmap *);
 	gi->deco.design.x = va_arg(ap, int);
@@ -1288,6 +1869,30 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
 	gi->callback_action = va_arg(ap, gadget_function);
 	break;
 
+      case GDI_COLOR_NR:
+	gi->colorpicker.nr = va_arg(ap, int);
+	break;
+
+      case GDI_COLOR_TYPE:
+	gi->colorpicker.type = va_arg(ap, int);
+	break;
+
+      case GDI_COLOR_VALUE:
+	gi->colorpicker.value = va_arg(ap, int);
+	break;
+
+      case GDI_COLOR_VALUES:
+	gi->colorpicker.values = va_arg(ap, int *);
+	break;
+
+      case GDI_COLOR_NAMES:
+	gi->colorpicker.names = va_arg(ap, char **);
+	break;
+
+      case GDI_COLOR_COUNT:
+	gi->colorpicker.count = va_arg(ap, int);
+	break;
+
       default:
 	Fail("HandleGadgetTags(): unknown tag %d", tag);
     }
@@ -1346,7 +1951,7 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
     Bitmap *src_bitmap;
     int src_x, src_y;
 
-    gi->width  = 2 * border_xsize + gi->textinput.size*font_width +button_size;
+    gi->width  = 2 * border_xsize + gi->textinput.size * font_width + button_size;
     gi->height = 2 * border_ysize + font_height;
 
     if (gi->selectbox.options == NULL)
@@ -1490,6 +2095,18 @@ static void HandleGadgetTags(struct GadgetInfo *gi, int first_tag, va_list ap)
     // always start with unselected text area (which is potentially cropped)
     gi->textarea.full_open = FALSE;
   }
+
+  if (gi->type & GD_TYPE_COLOR_PICKER)
+  {
+    gi->width  = CP_GADGET_WIDTH  + 2 * gi->border.xsize;
+    gi->height = CP_GADGET_HEIGHT + 2 * gi->border.ysize;
+
+    if (gi->colorpicker.count == 0)		// RGB colors
+      gi->colorpicker.color_hsv = get_hsv_from_int(gi->colorpicker.value);
+
+    // always start with closed color picker
+    gi->colorpicker.open = FALSE;
+  }
 }
 
 void ModifyGadget(struct GadgetInfo *gi, int first_tag, ...)
@@ -1594,6 +2211,11 @@ static void CheckRangeOfNumericInputGadget(struct GadgetInfo *gi)
 // global pointer to gadget actually in use (when mouse button pressed)
 static struct GadgetInfo *last_gi = NULL;
 
+boolean isMappedGadget(struct GadgetInfo *gi)
+{
+  return (gi != NULL && gi->mapped);
+}
+
 static void MapGadgetExt(struct GadgetInfo *gi, boolean redraw)
 {
   if (gi == NULL || gi->deactivated || gi->mapped)
@@ -1607,6 +2229,15 @@ static void MapGadgetExt(struct GadgetInfo *gi, boolean redraw)
 
   if (redraw)
     DrawGadget(gi, DG_UNPRESSED, DG_BUFFERED);
+
+  // when mapping color picker gadget, automatically activate it
+  if (gi->type & GD_TYPE_COLOR_PICKER)
+  {
+    if (redraw)
+      DrawGadget(gi, DG_PRESSED, DG_BUFFERED);
+
+    last_gi = gi;
+  }
 }
 
 void MapGadget(struct GadgetInfo *gi)
@@ -1701,6 +2332,11 @@ boolean anyScrollbarGadgetActive(void)
   return (last_gi && (last_gi->type & GD_TYPE_SCROLLBAR) && last_gi->mapped);
 }
 
+boolean anyColorPickerGadgetActive(void)
+{
+  return (last_gi && (last_gi->type & GD_TYPE_COLOR_PICKER) && last_gi->mapped);
+}
+
 boolean anyTextGadgetActive(void)
 {
   return (anyTextInputGadgetActive() ||
@@ -1822,13 +2458,14 @@ boolean HandleGadgets(int mx, int my, int button)
      new_gi->type & GD_TYPE_SELECTBOX && new_gi->selectbox.open &&
      insideSelectboxLine(new_gi, mx, my));
 
-  // if mouse button pressed outside text or selectbox gadget, deactivate it
-  if (anyTextGadgetActive() &&
+  // if mouse button pressed outside text, selectbox or color picker gadget, deactivate it
+  if ((anyTextGadgetActive() || anyColorPickerGadgetActive()) &&
       (gadget_pressed_off_borders ||
        (gadget_pressed_inside_select_line && !mouse_inside_select_area)))
   {
     struct GadgetInfo *gi = last_gi;
-    boolean gadget_changed = ((gi->event_mask & GD_EVENT_TEXT_LEAVING) != 0);
+    boolean gadget_changed = ((gi->event_mask & GD_EVENT_TEXT_LEAVING) != 0 ||
+                              (gi->event_mask & GD_EVENT_COLOR_PICKER_LEAVING) != 0);
 
     // check if text input gadget has changed its value
     if (gi->type & GD_TYPE_TEXT_INPUT)
@@ -1854,9 +2491,16 @@ boolean HandleGadgets(int mx, int my, int button)
     if (gi->type & GD_TYPE_SELECTBOX)
       gadget_changed = FALSE;
 
+    // color picker does not select color when closed by clicking outside
+    if (gi->type & GD_TYPE_COLOR_PICKER)
+      gi->colorpicker.value = -1;
+
     DrawGadget(gi, DG_UNPRESSED, gi->direct_draw);
 
-    gi->event.type = GD_EVENT_TEXT_LEAVING;
+    if (gi->type & GD_TYPE_COLOR_PICKER)
+      gi->event.type = GD_EVENT_COLOR_PICKER_LEAVING;
+    else
+      gi->event.type = GD_EVENT_TEXT_LEAVING;
 
     if (!(gi->type & GD_TYPE_SELECTBOX))
       DoGadgetCallbackAction(gi, gadget_changed);
@@ -1921,7 +2565,8 @@ boolean HandleGadgets(int mx, int my, int button)
       pressed_my = 0;
     }
     else if (!(gi->type & GD_TYPE_TEXT_INPUT ||
-	       gi->type & GD_TYPE_TEXT_AREA))	    // text input stays open
+	       gi->type & GD_TYPE_TEXT_AREA ||
+	       gi->type & GD_TYPE_COLOR_PICKER))    // text input and color picker stays open
       last_gi = NULL;
   }
 
@@ -2013,6 +2658,58 @@ boolean HandleGadgets(int mx, int my, int button)
       if (gi->selectbox.current_index != old_index)
 	DrawGadget(gi, DG_PRESSED, gi->direct_draw);
     }
+    else if (gi->type & GD_TYPE_COLOR_PICKER && button)
+    {
+      static int state = CP_STATE_NONE;
+      int xsize_main = CP_MAIN_GRADIENT_WIDTH;
+      int ysize_main = CP_MAIN_GRADIENT_HEIGHT;
+      int xsize_hue  = CP_HUE_GRADIENT_WIDTH;
+      int x = mx - gi->x - gi->border.xsize;
+      int y = my - gi->y - gi->border.ysize;
+      int x_hue  = MIN(MAX(0, x), xsize_hue - 1);
+      int x_main = MIN(MAX(0, x), xsize_main - 1);
+      int y_main = MIN(MAX(0, y), ysize_main - 1);
+
+      if (!motion_status)
+        state = (CP_INSIDE_MAIN(x, y)   ? CP_STATE_MAIN_GRADIENT_CHANGE :
+                 CP_INSIDE_HUE(x, y)    ? CP_STATE_HUE_GRADIENT_CHANGE :
+                 CP_INSIDE_SAMPLE(x, y) ? CP_STATE_SAMPLE_BOX :
+                 CP_STATE_NONE);
+
+      if (state == CP_STATE_MAIN_GRADIENT_CHANGE)
+      {
+        if (gi->colorpicker.count == 0)		// RGB colors
+        {
+          gi->colorpicker.color_hsv.s =       (double)x_main / xsize_main;
+          gi->colorpicker.color_hsv.v = 1.0 - (double)y_main / ysize_main;
+        }
+        else					// indexed colors
+        {
+          int count = gi->colorpicker.count;
+          int table_size = sqrt(count);
+          int box_size = CP_GADGET_WIDTH / table_size;
+
+          gi->colorpicker.value = (y_main / box_size * table_size + x_main / box_size);
+        }
+
+        DrawColorPicker(gi);
+      }
+      else if (state == CP_STATE_HUE_GRADIENT_CHANGE)
+      {
+        if (gi->colorpicker.count == 0)		// RGB colors
+        {
+          gi->colorpicker.color_hsv.h = (double)x_hue / xsize_hue * 360.0;
+
+          DrawColorPicker(gi);
+        }
+      }
+      else if (state == CP_STATE_SAMPLE_BOX)
+      {
+        SelectColorPickerColor(gi);
+
+        DoGadgetCallbackAction(gi, TRUE);
+      }
+    }
   }
 
   // handle gadget popup info text
@@ -2302,7 +2999,8 @@ boolean HandleGadgets(int mx, int my, int button)
 
     if (deactivate_gadget &&
 	!(gi->type & GD_TYPE_TEXT_INPUT ||
-	  gi->type & GD_TYPE_TEXT_AREA))	    // text input stays open
+	  gi->type & GD_TYPE_TEXT_AREA ||
+	  gi->type & GD_TYPE_COLOR_PICKER))	    // text input and color picker stays open
       DrawGadget(gi, DG_UNPRESSED, gi->direct_draw);
 
     gi->state = GD_BUTTON_UNPRESSED;
@@ -2355,7 +3053,8 @@ boolean HandleGadgetsKeyInput(Key key)
   if (gi == NULL || gi->deactivated || !gi->mapped ||
       !(gi->type & GD_TYPE_TEXT_INPUT ||
 	gi->type & GD_TYPE_TEXT_AREA ||
-	gi->type & GD_TYPE_SELECTBOX))
+	gi->type & GD_TYPE_SELECTBOX ||
+	gi->type & GD_TYPE_COLOR_PICKER))
     return FALSE;
 
   if (key == KSYM_Escape)
@@ -2394,8 +3093,23 @@ boolean HandleGadgetsKeyInput(Key key)
 
       StopTextInput();
     }
+    else if (anyColorPickerGadgetActive())
+    {
+      boolean gadget_changed = ((gi->event_mask & GD_EVENT_COLOR_PICKER_LEAVING) != 0);
+
+      // color picker does not select color when closed by Escape key
+      gi->colorpicker.value = -1;
+
+      DrawGadget(gi, DG_UNPRESSED, gi->direct_draw);
+
+      gi->event.type = GD_EVENT_COLOR_PICKER_LEAVING;
+
+      DoGadgetCallbackAction(gi, gadget_changed);
+
+      last_gi = NULL;
+    }
   }
-  else if (key == KSYM_Return)	// valid for both text input and selectbox
+  else if (key == KSYM_Return)	// valid for both text input, selectbox and color picker
   {
     boolean gadget_changed = ((gi->event_mask & GD_EVENT_TEXT_RETURN) != 0);
 
@@ -2424,6 +3138,12 @@ boolean HandleGadgetsKeyInput(Key key)
 
       DrawGadget(gi, DG_PRESSED, gi->direct_draw);
     }
+    else if (gi->type & GD_TYPE_COLOR_PICKER)
+    {
+      gadget_changed = ((gi->event_mask & GD_EVENT_COLOR_PICKER_LEAVING) != 0);
+
+      SelectColorPickerColor(gi);
+    }
     else
     {
       DrawGadget(gi, DG_UNPRESSED, gi->direct_draw);
@@ -2442,8 +3162,7 @@ boolean HandleGadgetsKeyInput(Key key)
     int cursor_pos = gi->textinput.cursor_position;
     char letter = getCharFromKey(key);
     boolean legal_letter = (gi->type == GD_TYPE_TEXT_INPUT_NUMERIC ?
-			    letter >= '0' && letter <= '9' :
-			    letter != 0);
+			    (letter >= '0' && letter <= '9') || letter == '-' : letter != 0);
 
     if (legal_letter && text_length < gi->textinput.size)
     {
