@@ -179,8 +179,22 @@ char *getImg3(const char *, const char *, const char *);
 char *getStringCopy(const char *);
 char *getStringCopyN(const char *, int);
 char *getStringCopyNStatic(const char *, int);
+char *getStringToUpper(const char *);
 char *getStringToLower(const char *);
+char *getStringPrint(char *, ...);
+void setStringPrint(char **, char *, ...);
+void appendStringPrint(char **, char *, ...);
 void setString(char **, const char *);
+char **getSplitStringArray(const char *s, const char *, int);
+int getStringArrayLength(char **);
+void freeStringArray(char **);
+char *getEscapedString(const char *);
+char *getUnescapedString(const char *);
+
+char *chugString(char *);
+char *chompString(char *);
+char *stripString(char *);
+
 boolean strEqual(const char *, const char *);
 boolean strEqualN(const char *, const char *, int);
 boolean strEqualCase(const char *, const char *);
@@ -193,13 +207,15 @@ boolean isURL(const char *);
 
 void GetOptions(int, char **,
 		void (*print_usage_function)(void),
-		void (*print_version_function)(void));
+		void (*print_version_function)(void),
+		void (*print_render_drivers_function)(void));
 
 void *checked_malloc(unsigned int);
 void *checked_calloc(unsigned int);
 void *checked_realloc(void *, unsigned int);
 void checked_free(void *);
 void clear_mem(void *, unsigned int);
+void *get_memcpy(const void *, size_t);
 
 void swap_numbers(int *, int *);
 void swap_number_pairs(int *, int *, int *, int *);
@@ -214,8 +230,10 @@ int putFile32BitInteger(FILE *, int, int);
 
 boolean getFileChunk(File *, char *, int *, int);
 int putFileChunk(FILE *, char *, int, int);
-int getFileVersion(File *);
-int putFileVersion(FILE *, int);
+VersionType getFileVersion(File *);
+VersionType getFileVersionExtended(File *);
+int putFileVersion(FILE *, VersionType);
+int putFileVersionExtended(FILE *, VersionType);
 
 void ReadBytesFromFile(File *, byte *, unsigned int);
 void WriteBytesToFile(FILE *, byte *, unsigned int);
@@ -254,7 +272,7 @@ char getValidConfigValueChar(char);
 
 int get_integer_from_string(char *);
 boolean get_boolean_from_string(char *);
-int get_switch3_from_string(char *);
+int get_switch_3_state_from_string(char *);
 int get_player_nr_from_string(char *);
 
 ListNode *newListNode(void);
@@ -263,7 +281,7 @@ void deleteNodeFromList(ListNode **, char *, void (*function)(void *));
 ListNode *getNodeFromKey(ListNode *, char *);
 int getNumNodes(ListNode *);
 
-File *openFile(char *, char *);
+File *openFile(const char *, const char *);
 int closeFile(File *);
 int checkEndOfFile(File *);
 size_t readFile(File *, void *, size_t, size_t);
@@ -271,8 +289,9 @@ size_t writeFile(File *, void *, size_t, size_t);
 int seekFile(File *, long, int);
 int getByteFromFile(File *);
 char *getStringFromFile(File *, char *, int);
-int copyFile(char *, char *);
-boolean touchFile(char *);
+int copyFile(const char *, const char *);
+boolean touchFile(const char *);
+size_t getSizeOfFile(const char *);
 
 Directory *openDirectory(char *);
 int closeDirectory(Directory *);
