@@ -83,6 +83,8 @@ struct SDLSurfaceInfo
   SDL_Surface *surface_masked;
   SDL_Texture *texture;
   SDL_Texture *texture_masked;
+
+  SDL_Renderer *renderer;	// renderer used to create textures
 };
 
 struct MouseCursorInfo
@@ -399,8 +401,13 @@ void SDLSetAlpha(SDL_Surface *, boolean, int);
 const char *SDLGetRendererName(void);
 boolean SDLSetNativeSurface(SDL_Surface **);
 SDL_Surface *SDLGetNativeSurface(SDL_Surface *);
+SDL_Surface *SDLCreateNativeSurface(int, int, int);
+Bitmap *SDLGetBitmapFromSurface_WithMaskedColor(SDL_Surface *, int, int, int);
+Bitmap *SDLGetBitmapFromSurface(SDL_Surface *);
 void SDLCreateBitmapTextures(Bitmap *);
+void SDLFreeBitmapSurfaces(Bitmap *);
 void SDLFreeBitmapTextures(Bitmap *);
+void SDLFreeBitmapPointers(Bitmap *);
 
 SDL_Surface *SDL_DisplayFormat(SDL_Surface *);
 void SDLSetWindowScaling(int);
@@ -415,13 +422,12 @@ void SDLSetScreenRenderingMode(char *);
 void SDLSetScreenVsyncMode(char *);
 void SDLRedrawWindow(void);
 void SDLSetWindowTitle(void);
+void SDLSetWindowResized(void);
 
 void SDLLimitScreenUpdates(boolean);
 void SDLInitVideoDisplay(void);
 void SDLInitVideoBuffer(boolean);
 boolean SDLSetVideoMode(boolean);
-void SDLCreateBitmapContent(Bitmap *, int, int, int);
-void SDLFreeBitmapPointers(Bitmap *);
 void SDLBlitSurface(SDL_Surface *, SDL_Surface *, int, int, int, int, int, int);
 void SDLCopyArea(Bitmap *, Bitmap *, int, int, int, int, int, int, int);
 void SDLBlitTexture(Bitmap *, int, int, int, int, int, int, int);
@@ -435,6 +441,7 @@ void SDLPutPixel(Bitmap *, int, int, Pixel);
 
 void SDLCopyInverseMasked(Bitmap *, Bitmap *, int, int, int, int, int, int);
 
+SDL_Surface *SDLZoomSurface(SDL_Surface *, int, int);
 Bitmap *SDLZoomBitmap(Bitmap *, int, int);
 
 Bitmap *SDLLoadImage(char *);
@@ -443,6 +450,7 @@ void SDLSetMouseCursor(struct MouseCursorInfo *);
 
 void SDLOpenAudio(void);
 void SDLCloseAudio(void);
+void SDLReopenAudio(void);
 
 void SDLWaitEvent(Event *);
 void SDLCorrectRawMousePosition(int *, int *);
