@@ -19,39 +19,41 @@
 // values for setup file handling
 #define TYPE_BOOLEAN			(1 << 0)
 #define TYPE_SWITCH			(1 << 1)
-#define TYPE_SWITCH3			(1 << 2)
+#define TYPE_SWITCH_3_STATES		(1 << 2)
 #define TYPE_YES_NO			(1 << 3)
 #define TYPE_YES_NO_AUTO		(1 << 4)
-#define TYPE_ECS_AGA			(1 << 5)
-#define TYPE_KEY			(1 << 6)
-#define TYPE_KEY_X11			(1 << 7)
-#define TYPE_INTEGER			(1 << 8)
-#define TYPE_STRING			(1 << 9)
-#define TYPE_PLAYER			(1 << 10)
-#define TYPE_ELEMENT			(1 << 11)
-#define TYPE_GRAPHIC			(1 << 12)
+#define TYPE_YES_NO_ASK			(1 << 5)
+#define TYPE_YES_NO_ONCE		(1 << 6)
+#define TYPE_ECS_AGA			(1 << 7)
+#define TYPE_KEY			(1 << 8)
+#define TYPE_KEY_X11			(1 << 9)
+#define TYPE_INTEGER			(1 << 10)
+#define TYPE_STRING			(1 << 11)
+#define TYPE_PLAYER			(1 << 12)
+#define TYPE_ELEMENT			(1 << 13)
+#define TYPE_GRAPHIC			(1 << 14)
 
 // additional values for setup screen
-#define TYPE_ENTER_SCREEN		(1 << 13)
-#define TYPE_LEAVE_SCREEN		(1 << 14)
-#define TYPE_ENTER_MENU			(1 << 15)
-#define TYPE_LEAVE_MENU			(1 << 16)
-#define TYPE_ENTER_LIST			(1 << 17)
-#define TYPE_LEAVE_LIST			(1 << 18)
-#define TYPE_TEXT_INPUT			(1 << 19)
-#define TYPE_EMPTY			(1 << 20)
-#define TYPE_SKIPPABLE			(1 << 21)
-#define TYPE_KEYTEXT			(1 << 22)
-#define TYPE_HEADLINE			(1 << 23)
+#define TYPE_ENTER_SCREEN		(1 << 15)
+#define TYPE_LEAVE_SCREEN		(1 << 16)
+#define TYPE_ENTER_MENU			(1 << 17)
+#define TYPE_LEAVE_MENU			(1 << 18)
+#define TYPE_ENTER_LIST			(1 << 19)
+#define TYPE_LEAVE_LIST			(1 << 20)
+#define TYPE_TEXT_INPUT			(1 << 21)
+#define TYPE_EMPTY			(1 << 22)
+#define TYPE_SKIPPABLE			(1 << 23)
+#define TYPE_KEYTEXT			(1 << 24)
+#define TYPE_HEADLINE			(1 << 25)
 
-#define TYPE_GHOSTED			(1 << 24)
-#define TYPE_QUERY			(1 << 25)
+#define TYPE_GHOSTED			(1 << 26)
+#define TYPE_QUERY			(1 << 27)
 
 // additional values for internal purposes
-#define TYPE_BITFIELD			(1 << 26)
-#define TYPE_CONTENT			(1 << 27)
-#define TYPE_ELEMENT_LIST		(1 << 28)
-#define TYPE_CONTENT_LIST		(1 << 29)
+#define TYPE_BITFIELD			(1 << 28)
+#define TYPE_CONTENT			(1 << 29)
+#define TYPE_ELEMENT_LIST		(1 << 30)
+#define TYPE_CONTENT_LIST		(1 << 31)
 
 // derived values for setup file handling
 #define TYPE_BOOLEAN_STYLE		(TYPE_BOOLEAN | \
@@ -61,8 +63,10 @@
 
 // derived values for setup screen
 #define TYPE_VALUE			(TYPE_BOOLEAN_STYLE	| \
-					 TYPE_SWITCH3		| \
+					 TYPE_SWITCH_3_STATES	| \
 					 TYPE_YES_NO_AUTO	| \
+					 TYPE_YES_NO_ASK	| \
+					 TYPE_YES_NO_ONCE	| \
 					 TYPE_KEY		| \
 					 TYPE_KEY_X11		| \
 					 TYPE_INTEGER		| \
@@ -277,9 +281,12 @@ char *getSetupFilename(void);
 char *getDefaultSetupFilename(void);
 char *getPlatformSetupFilename(void);
 char *getEditorSetupFilename(void);
+char *getFilenameFromCurrentLevelDirUpward(char *);
 char *getHelpAnimFilename(void);
 char *getHelpTextFilename(void);
 char *getLevelSetInfoFilename(int);
+char *getLevelInfoFilename(int);
+char *getLevelStoryFilename(int);
 char *getLevelSetTitleMessageFilename(int, boolean);
 char *getCreditsFilename(int, boolean);
 char *getProgramInfoFilename(int);
@@ -347,7 +354,7 @@ void InitUserDataDirectory(void);
 void SetFilePermissions(char *, int);
 
 void fprintFileHeader(FILE *, char *);
-int getFileVersionFromCookieString(const char *);
+VersionType getFileVersionFromCookieString(const char *);
 boolean checkCookieString(const char *, const char *);
 
 char *getFormattedSetupEntry(char *, char *);
@@ -365,16 +372,19 @@ SetupFileHash *newSetupFileHash(void);
 void freeSetupFileHash(SetupFileHash *);
 char *getHashEntry(SetupFileHash *, char *);
 void setHashEntry(SetupFileHash *, char *, char *);
-char *removeHashEntry(SetupFileHash *, char *);
+void removeHashEntry(SetupFileHash *, char *);
 SetupFileHash *loadSetupFileHash(char *);
 void setSetupInfo(struct TokenInfo *, int, char *);
 char *getSetupValue(int, void *);
 char *getSetupLine(struct TokenInfo *, char *, int);
 
-unsigned int get_hash_from_key(void *);
-int hash_keys_are_equal(void *, void *);
+unsigned int get_hash_from_string(void *);
+unsigned int get_hash_from_integer(void *);
+int hash_key_strings_are_equal(void *, void *);
+int hash_key_integers_are_equal(void *, void *);
 
 int GetZipFileTreeType(char *);
+boolean CheckZipFileForDirectory(char *, char *, int);
 char *ExtractZipFileIntoDirectory(char *, char *, int);
 
 boolean AdjustGraphicsForEMC(void);
@@ -397,7 +407,9 @@ boolean CreateUserLevelSet(char *, char *, char *, int, boolean);
 
 void UpdateLastPlayedLevels_TreeInfo(void);
 void StoreLastPlayedLevels(TreeInfo *);
+void ForcedStoreLastPlayedLevels(TreeInfo *);
 void RestoreLastPlayedLevels(TreeInfo **);
+boolean CheckLastPlayedLevels(void);
 
 void LoadLevelSetup_LastSeries(void);
 void SaveLevelSetup_LastSeries(void);
