@@ -41,6 +41,7 @@ struct ArtworkInfo	artwork;
 struct JoystickInfo	joystick;
 struct SetupInfo	setup;
 struct UserInfo		user;
+struct ZipFileInfo	zip_file;
 
 LevelDirTree	       *leveldir_first_all = NULL;
 LevelDirTree	       *leveldir_first = NULL;
@@ -73,7 +74,7 @@ void InitProgramInfo(char *command_filename,
 		     char *config_filename, char *userdata_subdir,
 		     char *program_basename, char *program_title,
 		     char *icon_filename, char *cookie_prefix,
-		     char *program_version_string, int program_version)
+		     char *program_version_string, VersionType program_version)
 {
   program.command_basepath = getBasePath(command_filename);
   program.command_basename = getBaseName(command_filename);
@@ -121,7 +122,7 @@ void InitNetworkInfo(boolean enabled, boolean connected, boolean serveronly,
   network.is_server_thread = FALSE;
 }
 
-void InitRuntimeInfo()
+void InitRuntimeInfo(void)
 {
 #if defined(HAS_TOUCH_DEVICE)
   runtime.uses_touch_device = TRUE;
@@ -139,6 +140,11 @@ void SetWindowTitle(void)
   SDLSetWindowTitle();
 }
 
+void SetWindowResized(void)
+{
+  SDLSetWindowResized();
+}
+
 void InitWindowTitleFunction(char *(*window_title_function)(void))
 {
   program.window_title_function = window_title_function;
@@ -453,6 +459,11 @@ void SetDrawBackgroundMask(int draw_background_mask)
   gfx.draw_background_mask = draw_background_mask;
 }
 
+int GetDrawBackgroundMask(void)
+{
+  return gfx.draw_background_mask;
+}
+
 void SetBackgroundBitmap(Bitmap *background_bitmap_tile, int mask,
 			 int x, int y, int width, int height)
 {
@@ -624,7 +635,7 @@ Bitmap *CreateBitmap(int width, int height, int depth)
   int real_height = MAX(1, height);	// prevent zero bitmap height
   int real_depth  = GetRealDepth(depth);
 
-  SDLCreateBitmapContent(new_bitmap, real_width, real_height, real_depth);
+  new_bitmap->surface = SDLCreateNativeSurface(real_width, real_height, real_depth);
 
   new_bitmap->width  = real_width;
   new_bitmap->height = real_height;
@@ -938,6 +949,12 @@ void BlitBitmapMasked(Bitmap *src_bitmap, Bitmap *dst_bitmap,
 		      int src_x, int src_y, int width, int height,
 		      int dst_x, int dst_y)
 {
+  if (program.headless)
+    return;
+
+  if (src_bitmap == NULL || dst_bitmap == NULL)
+    return;
+
   if (DrawingDeactivated(dst_x, dst_y))
     return;
 
@@ -1376,13 +1393,27 @@ static void CreateScaledBitmaps(Bitmap **bitmaps, int zoom_factor,
     bitmaps[IMG_BITMAP_2x2]   = tmp_bitmap_16;
     bitmaps[IMG_BITMAP_1x1]   = tmp_bitmap_32;
 
-    if (width_0 != width_1)
+    // store the "game" bitmap (with game tile sized graphics), if not already stored
+
+    int tmp_bitmap_0_nr = -1;
+
+    for (i = 0; i < NUM_IMG_BITMAPS; i++)
+      if (bitmaps[i] == tmp_bitmap_0)
+	tmp_bitmap_0_nr = i;
+
+    if (tmp_bitmap_0_nr == -1)	// game tile size bitmap not stored
+    {
+      // store pointer of game tile size bitmap (not used for any other size)
       bitmaps[IMG_BITMAP_CUSTOM] = tmp_bitmap_0;
 
-    if (bitmaps[IMG_BITMAP_CUSTOM])
+      // set game bitmap pointer to game tile size bitmap of other size
       bitmaps[IMG_BITMAP_PTR_GAME] = bitmaps[IMG_BITMAP_CUSTOM];
+    }
     else
-      bitmaps[IMG_BITMAP_PTR_GAME] = bitmaps[IMG_BITMAP_STANDARD];
+    {
+      // set game bitmap pointer to corresponding sized bitmap
+      bitmaps[IMG_BITMAP_PTR_GAME] = bitmaps[tmp_bitmap_0_nr];
+    }
 
     // store the "final" (up-scaled) original bitmap, if not already stored
 
@@ -1662,6 +1693,8 @@ void OpenAudio(void)
   audio.device_name = NULL;
   audio.device_fd = -1;
 
+  audio.sample_rate = DEFAULT_AUDIO_SAMPLE_RATE;
+
   audio.num_channels = 0;
   audio.music_channel = 0;
   audio.first_sound_channel = 0;
@@ -1835,6 +1868,28 @@ void PushUserEvent(int code, int value1, int value2)
   SDL_PushEvent((SDL_Event *)&event);
 }
 
+void PushDropEvent(char *file)
+{
+  SDL_DropEvent event;
+
+  SDL_memset(&event, 0, sizeof(event));
+
+  event.type = SDL_DROPBEGIN;
+  event.file = NULL;
+
+  SDL_PushEvent((SDL_Event *)&event);
+
+  event.type = SDL_DROPFILE;
+  event.file = getStringCopy(file);
+
+  SDL_PushEvent((SDL_Event *)&event);
+
+  event.type = SDL_DROPCOMPLETE;
+  event.file = NULL;
+
+  SDL_PushEvent((SDL_Event *)&event);
+}
+
 boolean PendingEscapeKeyEvent(void)
 {
   if (PendingEvent())
