@@ -305,8 +305,8 @@ static void DrawVideoDisplay_DateTime(unsigned int state, unsigned int value)
 	pos->y == -1)
       continue;
 
-    xpos = VX + pos->x + (type & DATETIME_XOFFSET_1 ? pos->xoffset  :
-			  type & DATETIME_XOFFSET_2 ? pos->xoffset2 : 0);
+    xpos = VX + pos->x + ((type & DATETIME_XOFFSET_1) ? pos->xoffset  :
+			  (type & DATETIME_XOFFSET_2) ? pos->xoffset2 : 0);
     ypos = VY + pos->y;
 
     if ((type & DATETIME_DATE) && (state & VIDEO_STATE_DATE_ON))
@@ -319,11 +319,11 @@ static void DrawVideoDisplay_DateTime(unsigned int state, unsigned int value)
       int month = month_index + 1;
       int day = value % 100;
 
-      strcpy(s, (type & DATETIME_DATE_YYYY ? int2str(year4, 4) :
-		 type & DATETIME_DATE_YY   ? int2str(year2, 2) :
-		 type & DATETIME_DATE_MON  ? month_shortnames[month_index] :
-		 type & DATETIME_DATE_MM   ? int2str(month, 2) :
-		 type & DATETIME_DATE_DD   ? int2str(day, 2) : ""));
+      strcpy(s, ((type & DATETIME_DATE_YYYY) ? int2str(year4, 4) :
+		 (type & DATETIME_DATE_YY)   ? int2str(year2, 2) :
+		 (type & DATETIME_DATE_MON)  ? month_shortnames[month_index] :
+		 (type & DATETIME_DATE_MM)   ? int2str(month, 2) :
+		 (type & DATETIME_DATE_DD)   ? int2str(day, 2) : ""));
 
       DrawText(xpos, ypos, s, pos->font);
     }
@@ -335,10 +335,10 @@ static void DrawVideoDisplay_DateTime(unsigned int state, unsigned int value)
       int mm = (value / 60) % 60;
       int ss = value % 60;
 
-      strcpy(s, (type & DATETIME_TIME_HH  ? int2str(hh, 2) :
-		 type & DATETIME_TIME_MIN ? int2str(min, 2) :
-		 type & DATETIME_TIME_MM  ? int2str(mm, 2) :
-		 type & DATETIME_TIME_SS  ? int2str(ss, 2) : ""));
+      strcpy(s, ((type & DATETIME_TIME_HH)  ? int2str(hh, 2) :
+		 (type & DATETIME_TIME_MIN) ? int2str(min, 2) :
+		 (type & DATETIME_TIME_MM)  ? int2str(mm, 2) :
+		 (type & DATETIME_TIME_SS)  ? int2str(ss, 2) : ""));
 
       DrawText(xpos, ypos, s, pos->font);
     }
@@ -404,6 +404,23 @@ static void DrawVideoDisplayCurrentState(void)
   DrawVideoDisplaySymbol(state);
 }
 
+static void DrawVideoDisplayCurrentState_PlayAction(boolean update_draw_label_on)
+{
+  int state = 0;
+
+  if (tape.warp_forward)
+    state |= VIDEO_STATE_WARP(update_draw_label_on);
+  else if (tape.fast_forward)
+    state |= VIDEO_STATE_FFWD(update_draw_label_on);
+
+  if (tape.pause_before_end)
+    state |= VIDEO_STATE_PBEND(update_draw_label_on);
+
+  // draw labels and symbols separately to prevent labels overlapping symbols
+  DrawVideoDisplayLabel(state);
+  DrawVideoDisplaySymbol(state);
+}
+
 void DrawCompleteVideoDisplay(void)
 {
   struct GraphicInfo *g_tape = &graphic_info[IMG_BACKGROUND_TAPE];
@@ -481,9 +498,14 @@ struct AutoPlayInfo
   int level_nr;
   int num_levels_played;
   int num_levels_solved;
+  int num_levels_failed;
+  int num_levels_broken;
+  int num_levels_unsolv;
   int num_tapes_patched;
   int num_tape_missing;
   boolean level_failed[MAX_TAPES_PER_SET];
+  boolean level_broken[MAX_TAPES_PER_SET];
+  boolean level_unsolv[MAX_TAPES_PER_SET];
   char *tape_filename;
 };
 
@@ -592,10 +614,31 @@ static void PrintTapeReplaySummary(struct AutoPlayInfo *autoplay)
 
     if (autoplay->num_levels_played != autoplay->num_levels_solved)
     {
-      Print(", FAILED:");
-      for (i = 0; i < MAX_TAPES_PER_SET; i++)
-	if (autoplay->level_failed[i])
-	  Print(" %03d", i);
+      // not all tapes ran successfully -- show which tapes failed, are broken or are unsolvable
+
+      if (autoplay->num_levels_failed > 0)
+      {
+        Print(", FAILED:");
+        for (i = 0; i < MAX_TAPES_PER_SET; i++)
+          if (autoplay->level_failed[i])
+            Print(" %03d", i);
+      }
+
+      if (autoplay->num_levels_broken > 0)
+      {
+        Print(", BROKEN:");
+        for (i = 0; i < MAX_TAPES_PER_SET; i++)
+          if (autoplay->level_broken[i])
+            Print(" %03d", i);
+      }
+
+      if (autoplay->num_levels_unsolv > 0)
+      {
+        Print(", UNSOLVABLE:");
+        for (i = 0; i < MAX_TAPES_PER_SET; i++)
+          if (autoplay->level_unsolv[i])
+            Print(" %03d", i);
+      }
     }
   }
 
@@ -629,6 +672,30 @@ static void CloseTapeLogfile(void)
 // tape control functions
 // ============================================================================
 
+void TapeSetDateFromIsoDateString(char *date)
+{
+  int i;
+
+  // check ISO date string for correct length
+  if (strlen(date) != 10)
+    return;
+
+  // check ISO date string for correct format
+  for (i = 0; i < strlen(date); i++)
+    if (((i != 4 && i != 7) && (date[i] < '0' || date[i] > '9')) ||
+	((i == 4 || i == 7) && (date[i] != '-')))
+      return;
+
+  int yy = (date[2] - '0') * 10 + (date[3] - '0');
+  int mm = (date[5] - '0') * 10 + (date[6] - '0');
+  int dd = (date[8] - '0') * 10 + (date[9] - '0');
+
+  if (mm < 1 || mm > 12 || dd < 1 || dd > 31)
+    return;
+
+  tape.date = 10000 * yy + 100 * (mm - 1) + dd;
+}
+
 void TapeSetDateFromEpochSeconds(time_t epoch_seconds)
 {
   struct tm *lt = localtime(&epoch_seconds);
@@ -664,15 +731,20 @@ void TapeErase(void)
   tape.level_nr = level_nr;
   tape.pos[tape.counter].delay = 0;
   tape.changed = TRUE;
+  tape.solved = FALSE;
 
   tape.random_seed = InitRND(level.random_seed);
 
-  tape.file_version = FILE_VERSION_ACTUAL;
-  tape.game_version = GAME_VERSION_ACTUAL;
-  tape.engine_version = level.game_version;
+  tape.file_version        = FILE_VERSION_ACTUAL;
+  tape.game_version        = GAME_VERSION_ACTUAL;
+  tape.game_version_full   = GAME_VERSION_ACTUAL_FULL;
+  tape.engine_version      = level.game_version;
+  tape.engine_version_full = level.game_version;
 
   tape.property_bits = TAPE_PROPERTY_NONE;
 
+  tape.bd_replay = FALSE;
+
   TapeSetDateFromNow();
 
   for (i = 0; i < MAX_PLAYERS; i++)
@@ -760,6 +832,7 @@ static void TapeAppendRecording(void)
   // start recording
   tape.recording = TRUE;
   tape.changed = TRUE;
+  tape.solved = FALSE;
 
   // set current delay (for last played move)
   tape.pos[tape.counter].delay = tape.delay_played;
@@ -777,7 +850,9 @@ static void TapeAppendRecording(void)
 
 void TapeHaltRecording(void)
 {
-  tape.counter++;
+  // only advance tape counter if any input events have been recorded
+  if (tape.pos[tape.counter].delay > 0)
+    tape.counter++;
 
   // initialize delay for next tape entry (to be able to continue recording)
   if (tape.counter < MAX_TAPE_LEN)
@@ -832,6 +907,8 @@ boolean TapeAddAction(byte action[MAX_TAPE_ACTIONS])
     tape.pos[tape.counter].delay++;
   }
 
+  tape.changed = TRUE;
+
   return TRUE;
 }
 
@@ -843,6 +920,9 @@ void TapeRecordAction(byte action_raw[MAX_TAPE_ACTIONS])
   if (!tape.recording)		// (record action even when tape is paused)
     return;
 
+  if (!checkGameRunning())
+    return;
+
   for (i = 0; i < MAX_TAPE_ACTIONS; i++)
     action[i] = action_raw[i];
 
@@ -930,6 +1010,10 @@ void TapeTogglePause(boolean toggle_mode)
 
     ModifyPauseButtons();
   }
+
+  // stop tape when leaving auto-pause after completely replaying tape
+  if (tape.playing && !tape.pausing && tape.counter >= tape.length)
+    TapeStop();
 }
 
 void TapeStartPlaying(void)
@@ -974,7 +1058,7 @@ void TapeStopPlaying(void)
   MapTapeEjectButton();
 }
 
-byte *TapePlayAction(void)
+byte *TapePlayActionExt(boolean bd_replay)
 {
   int update_delay = FRAMES_PER_SECOND / 2;
   boolean update_video_display = (FrameCounter % update_delay == 0);
@@ -985,6 +1069,18 @@ byte *TapePlayAction(void)
   if (!tape.playing || tape.pausing)
     return NULL;
 
+  if (!checkGameRunning())
+    return NULL;
+
+  if (tape.bd_replay && !bd_replay)
+  {
+    // skip tape records without game actions for native BD tapes (but update tape display)
+    if (update_video_display && !tape.deactivate_display)
+      DrawVideoDisplayCurrentState_PlayAction(update_draw_label_on);
+
+    return NULL;
+  }
+
   if (tape.pause_before_end)  // stop some seconds before end of tape
   {
     if (TapeTime > (int)tape.length_seconds - TAPE_PAUSE_SECONDS_BEFORE_DEATH)
@@ -992,16 +1088,22 @@ byte *TapePlayAction(void)
       TapeStopWarpForward();
       TapeTogglePause(TAPE_TOGGLE_MANUAL);
 
+      if (setup.autorecord_after_replay)
+	TapeAppendRecording();
+
       return NULL;
     }
   }
 
   if (tape.counter >= tape.length)	// end of tape reached
   {
-    if (tape.warp_forward && !tape.auto_play)
+    if (!tape.auto_play)
     {
       TapeStopWarpForward();
       TapeTogglePause(TAPE_TOGGLE_MANUAL);
+
+      if (setup.autorecord_after_replay)
+	TapeAppendRecording();
     }
     else
     {
@@ -1012,21 +1114,7 @@ byte *TapePlayAction(void)
   }
 
   if (update_video_display && !tape.deactivate_display)
-  {
-    int state = 0;
-
-    if (tape.warp_forward)
-      state |= VIDEO_STATE_WARP(update_draw_label_on);
-    else if (tape.fast_forward)
-      state |= VIDEO_STATE_FFWD(update_draw_label_on);
-
-    if (tape.pause_before_end)
-      state |= VIDEO_STATE_PBEND(update_draw_label_on);
-
-    // draw labels and symbols separately to prevent labels overlapping symbols
-    DrawVideoDisplayLabel(state);
-    DrawVideoDisplaySymbol(state);
-  }
+    DrawVideoDisplayCurrentState_PlayAction(update_draw_label_on);
 
   for (i = 0; i < MAX_TAPE_ACTIONS; i++)
     action[i] = tape.pos[tape.counter].action[i];
@@ -1057,7 +1145,7 @@ byte *TapePlayAction(void)
   }
 
   tape.delay_played++;
-  if (tape.delay_played >= tape.pos[tape.counter].delay)
+  if (tape.delay_played >= tape.pos[tape.counter].delay || tape.bd_replay)
   {
     tape.counter++;
     tape.delay_played = 0;
@@ -1072,6 +1160,34 @@ byte *TapePlayAction(void)
   return action;
 }
 
+byte *TapePlayAction_BD(void)
+{
+  return TapePlayActionExt(TRUE);
+}
+
+byte *TapePlayAction(void)
+{
+  return TapePlayActionExt(FALSE);
+}
+
+byte *TapeCorrectAction_BD(byte *action)
+{
+  if (tape.playing)
+  {
+    // only read next tape action if not playing native BD replay
+    if (!TapeIsPlaying_ReplayBD())
+      action = TapePlayAction();
+  }
+  else if (tape.recording)
+  {
+    byte tape_action[MAX_TAPE_ACTIONS] = { action[0] };
+
+    TapeRecordAction(tape_action);
+  }
+
+  return action;
+}
+
 void TapeStop(void)
 {
   if (tape.pausing)
@@ -1177,22 +1293,25 @@ static void TapeSingleStep(void)
 
 void TapeQuickSave(void)
 {
-  if (game_status == GAME_MODE_MAIN)
+  if (game_status != GAME_MODE_PLAYING)
   {
-    Request("No game that can be saved!", REQ_CONFIRM);
+    Request("No game that could be saved!", REQ_CONFIRM);
 
     return;
   }
 
-  if (game_status != GAME_MODE_PLAYING)
-    return;
+  if (!tape.recording)
+  {
+    Request("No recording that could be saved!", REQ_CONFIRM);
 
-  if (tape.recording)
-    TapeHaltRecording();	// prepare tape for saving on-the-fly
+    return;
+  }
+
+  TapeHaltRecording();		// prepare tape for saving on-the-fly
 
   if (TAPE_IS_EMPTY(tape))
   {
-    Request("No tape that can be saved!", REQ_CONFIRM);
+    Request("No tape that could be saved!", REQ_CONFIRM);
 
     return;
   }
@@ -1294,7 +1413,7 @@ static boolean checkRestartGame(char *message)
 
 void TapeRestartGame(void)
 {
-  if (score_info_tape_play)
+  if (tape.playing)
   {
     TapeStartGamePlaying();
 
@@ -1330,11 +1449,18 @@ void TapeReplayAndPauseBeforeEnd(void)
   tape.quick_resume = TRUE;
 }
 
+boolean TapeIsPlaying_ReplayBD(void)
+{
+  return (tape.playing && tape.bd_replay);
+}
+
 boolean hasSolutionTape(void)
 {
   boolean tape_file_exists = fileExists(getSolutionTapeFilename(level_nr));
-  boolean level_has_tape = (level.game_engine_type == GAME_ENGINE_TYPE_SP &&
-			    level.native_sp_level->demo.is_available);
+  boolean level_has_tape = ((level.game_engine_type == GAME_ENGINE_TYPE_BD &&
+			     level.native_bd_level->replay != NULL) ||
+			    (level.game_engine_type == GAME_ENGINE_TYPE_SP &&
+			     level.native_sp_level->demo.is_available));
 
   return (tape_file_exists || level_has_tape);
 }
@@ -1380,15 +1506,14 @@ boolean PlaySolutionTape(void)
 
 static boolean PlayScoreTape_WaitForDownload(void)
 {
-  unsigned int download_delay = 0;
-  unsigned int download_delay_value = 10000;
+  DelayCounter download_delay = { 10000 };
 
   ResetDelayCounter(&download_delay);
 
   // wait for score tape to be successfully downloaded (and fail on timeout)
   while (!server_scores.tape_downloaded)
   {
-    if (DelayReached(&download_delay, download_delay_value))
+    if (DelayReached(&download_delay))
       return FALSE;
 
     UPDATE_BUSY_STATE_NOT_LOADING();
@@ -1578,15 +1703,14 @@ static void AutoPlayTapes_SetScoreEntry(int score, int time)
 
 static boolean AutoPlayTapes_WaitForUpload(void)
 {
-  unsigned int upload_delay = 0;
-  unsigned int upload_delay_value = 10000;
+  DelayCounter upload_delay = { 10000 };
 
   ResetDelayCounter(&upload_delay);
 
   // wait for score tape to be successfully uploaded (and fail on timeout)
   while (!server_scores.uploaded)
   {
-    if (DelayReached(&upload_delay, upload_delay_value))
+    if (DelayReached(&upload_delay))
     {
       PrintNoLog("\r");
       Print("- uploading score tape to score server - TIMEOUT.\n");
@@ -1618,30 +1742,34 @@ static int AutoPlayTapesExt(boolean initialize)
     "original tape",
     "em_random_bug",
     "screen_34x34",
+    "bd_old_engine",
 
     NULL
   };
-  static int patch_version_first[] =
+  static VersionType patch_version_first[] =
   {
     VERSION_IDENT(0,0,0,0),
     VERSION_IDENT(3,3,1,0),
     VERSION_IDENT(0,0,0,0),
+    VERSION_IDENT(4,4,0,0),
 
     -1
   };
-  static int patch_version_last[] =
+  static VersionType patch_version_last[] =
   {
     VERSION_IDENT(9,9,9,9),
     VERSION_IDENT(4,0,1,1),
     VERSION_IDENT(4,2,2,0),
+    VERSION_IDENT(4,4,0,0),
 
     -1
   };
-  static byte patch_property_bit[] =
+  static unsigned short patch_property_bit[] =
   {
     TAPE_PROPERTY_NONE,
     TAPE_PROPERTY_EM_RANDOM_BUG,
     TAPE_PROPERTY_NONE,
+    TAPE_PROPERTY_BD_OLD_ENGINE,
 
     -1
   };
@@ -1724,11 +1852,42 @@ static int AutoPlayTapesExt(boolean initialize)
     if (patch_nr == 0)
       autoplay.num_levels_played++;
 
+    boolean level_failed = FALSE;
+    boolean level_broken = FALSE;
+    boolean level_unsolv = FALSE;
+
     if (tape.auto_play_level_solved)
+    {
       autoplay.num_levels_solved++;
+    }
+    else
+    {
+      if (tape.solved)
+      {
+        // tape is marked as solution tape, but fails to solve level when replayed
+        autoplay.num_levels_broken++;
+        level_broken = TRUE;
+      }
+      else if (tape.game_version < VERSION_IDENT(4,3,2,3))
+      {
+        // tape fails to solve level when replayed (tape is either broken or unsolvable)
+        autoplay.num_levels_failed++;
+        level_failed = TRUE;
+      }
+      else
+      {
+        // tape is explicitly marked as not being a solution tape in newer versions
+        autoplay.num_levels_unsolv++;
+        level_unsolv = TRUE;
+      }
+    }
 
     if (level_nr >= 0 && level_nr < MAX_TAPES_PER_SET)
-      autoplay.level_failed[level_nr] = !tape.auto_play_level_solved;
+    {
+      autoplay.level_failed[level_nr] = level_failed;
+      autoplay.level_broken[level_nr] = level_broken;
+      autoplay.level_unsolv[level_nr] = level_unsolv;
+    }
   }
   else
   {
@@ -1837,7 +1996,11 @@ static int AutoPlayTapesExt(boolean initialize)
       autoplay.num_tape_missing = 0;
 
       for (i = 0; i < MAX_TAPES_PER_SET; i++)
+      {
 	autoplay.level_failed[i] = FALSE;
+	autoplay.level_broken[i] = FALSE;
+	autoplay.level_unsolv[i] = FALSE;
+      }
 
       PrintTapeReplayHeader(&autoplay);
 
@@ -1971,10 +2134,10 @@ static int AutoPlayTapesExt(boolean initialize)
 	      level_nr,  tape_patch_info,
 	      tape.length_seconds / 60, tape.length_seconds % 60,
 	      "not suitable for version",
-	      (tape.engine_version / 1000000) % 100,
-	      (tape.engine_version / 10000  ) % 100,
-	      (tape.engine_version / 100    ) % 100,
-	      (tape.engine_version          ) % 100);
+	      VERSION_SUPER(tape.engine_version),
+	      VERSION_MAJOR(tape.engine_version),
+	      VERSION_MINOR(tape.engine_version),
+	      VERSION_PATCH(tape.engine_version));
 
 	skip_patch = TRUE;
       }
@@ -2121,21 +2284,21 @@ int AutoPlayTapesContinue(void)
 static boolean PatchTape(struct TapeInfo *tape, char *mode)
 {
   Print("[%d.%d.%d.%d]: ",
-	(tape->engine_version / 1000000) % 100,
-	(tape->engine_version / 10000  ) % 100,
-	(tape->engine_version / 100    ) % 100,
-	(tape->engine_version          ) % 100);
+	VERSION_SUPER(tape->engine_version),
+	VERSION_MAJOR(tape->engine_version),
+	VERSION_MINOR(tape->engine_version),
+	VERSION_PATCH(tape->engine_version));
 
   if (strEqual(mode, "info"))
   {
-    Print("property bits == 0x%02x\n", tape->property_bits);
+    Print("property bits == 0x%04x\n", tape->property_bits);
 
     return FALSE;
   }
 
   boolean unpatch_tape = FALSE;
   boolean use_property_bit = FALSE;
-  byte property_bitmask = 0;
+  unsigned short property_bitmask = 0;
 
   if (strSuffix(mode, ":0") ||
       strSuffix(mode, ":off") ||
@@ -2190,6 +2353,20 @@ static boolean PatchTape(struct TapeInfo *tape, char *mode)
     tape->scr_fieldx = scr_fieldx_new;
     tape->scr_fieldy = scr_fieldy_new;
   }
+  else if (strEqual(mode, "bd_old_engine") || strPrefix(mode, "bd_old_engine:"))
+  {
+    // this problem was introduced and fixed in version 4.4.0.0 pre-releases
+    if (tape->engine_version != VERSION_IDENT(4,4,0,0))
+    {
+      Print("This tape version cannot be patched against the old BD engine problem!\n");
+
+      return FALSE;
+    }
+
+    property_bitmask = TAPE_PROPERTY_BD_OLD_ENGINE;
+
+    use_property_bit = TRUE;
+  }
   else
   {
     Print("Unknown patch mode '%s'!\n", mode);
@@ -2200,7 +2377,7 @@ static boolean PatchTape(struct TapeInfo *tape, char *mode)
   // patching tapes using property bits may be used for several patch modes
   if (use_property_bit)
   {
-    byte property_bits = tape->property_bits;
+    unsigned short property_bits = tape->property_bits;
     boolean set_property_bit = (unpatch_tape ? FALSE : TRUE);
 
     if (set_property_bit)
@@ -2237,6 +2414,7 @@ void PatchTapes(void)
     Print("Supported patch modes:\n");
     Print("- \"em_random_bug\"   - use 64-bit random value bug for EM engine\n");
     Print("- \"screen_34x34\"    - force visible playfield size of 34 x 34\n");
+    Print("- \"bd_old_engine\"   - use old game engine version for BD engine\n");
     PrintLine("-", 79);
     Print("Supported modifiers:\n");
     Print("- add \":0\", \":off\" or \":clear\" to patch mode to un-patch tape file\n");
