@@ -17,51 +17,50 @@
 
 
 // for DrawElementShifted
-#define NO_CUTTING		0
-#define CUT_ABOVE		(1 << 0)
-#define CUT_BELOW		(1 << 1)
-#define CUT_LEFT		(1 << 2)
-#define CUT_RIGHT		(1 << 3)
+#define NO_CUTTING			0
+#define CUT_ABOVE			(1 << 0)
+#define CUT_BELOW			(1 << 1)
+#define CUT_LEFT			(1 << 2)
+#define CUT_RIGHT			(1 << 3)
 
 // for masking functions
-#define NO_MASKING		0
-#define USE_MASKING		1
+#define NO_MASKING			0
+#define USE_MASKING			1
  
 // for MoveDoor
-#define DOOR_OPEN_1		(1 << 0)
-#define DOOR_OPEN_2		(1 << 1)
-#define DOOR_CLOSE_1		(1 << 2)
-#define DOOR_CLOSE_2		(1 << 3)
-#define DOOR_OPEN_ALL		(DOOR_OPEN_1 | DOOR_OPEN_2)
-#define DOOR_CLOSE_ALL		(DOOR_CLOSE_1 | DOOR_CLOSE_2)
-#define DOOR_ACTION_1		(DOOR_OPEN_1 | DOOR_CLOSE_1)
-#define DOOR_ACTION_2		(DOOR_OPEN_2 | DOOR_CLOSE_2)
-#define DOOR_ACTION		(DOOR_ACTION_1 | DOOR_ACTION_2)
-#define DOOR_COPY_BACK		(1 << 4)
-#define DOOR_NO_COPY_BACK	(1 << 5)
-#define DOOR_NO_DELAY		(1 << 6)
-#define DOOR_FORCE_ANIM		(1 << 7)
-#define DOOR_FORCE_REDRAW	(1 << 8)
-#define DOOR_GET_STATE		(1 << 9)
-#define DOOR_SET_STATE		(1 << 10)
+#define DOOR_OPEN_1			(1 << 0)
+#define DOOR_OPEN_2			(1 << 1)
+#define DOOR_CLOSE_1			(1 << 2)
+#define DOOR_CLOSE_2			(1 << 3)
+#define DOOR_OPEN_ALL			(DOOR_OPEN_1 | DOOR_OPEN_2)
+#define DOOR_CLOSE_ALL			(DOOR_CLOSE_1 | DOOR_CLOSE_2)
+#define DOOR_ACTION_1			(DOOR_OPEN_1 | DOOR_CLOSE_1)
+#define DOOR_ACTION_2			(DOOR_OPEN_2 | DOOR_CLOSE_2)
+#define DOOR_ACTION			(DOOR_ACTION_1 | DOOR_ACTION_2)
+#define DOOR_COPY_BACK			(1 << 4)
+#define DOOR_NO_COPY_BACK		(1 << 5)
+#define DOOR_NO_DELAY			(1 << 6)
+#define DOOR_FORCE_ANIM			(1 << 7)
+#define DOOR_FORCE_REDRAW		(1 << 8)
+#define DOOR_GET_STATE			(1 << 9)
+#define DOOR_SET_STATE			(1 << 10)
 
-#define DOOR_1			(DOOR_ACTION_1)
-#define DOOR_2			(DOOR_ACTION_2)
-#define DOOR_OPEN		(DOOR_OPEN_ALL)
-#define DOOR_CLOSE		(DOOR_CLOSE_ALL)
+#define DOOR_1				(DOOR_ACTION_1)
+#define DOOR_2				(DOOR_ACTION_2)
+#define DOOR_OPEN			(DOOR_OPEN_ALL)
+#define DOOR_CLOSE			(DOOR_CLOSE_ALL)
 
 #define DOOR_INDEX_FROM_TOKEN(x)	((x) == DOOR_1 ? 0 : 1)
 #define DOOR_TOKEN_FROM_INDEX(x)	((x) == 0 ? DOOR_1 ? : DOOR_2)
-#define REDRAW_DOOR_FROM_TOKEN(x)	((x) == DOOR_1 ? REDRAW_DOOR_1 : \
-					 REDRAW_DOOR_2)
+#define REDRAW_DOOR_FROM_TOKEN(x)	((x) == DOOR_1 ? REDRAW_DOOR_1 : REDRAW_DOOR_2)
 
 // for Request
-#define REQ_ASK			(1 << 0)
-#define REQ_CONFIRM		(1 << 1)
-#define REQ_PLAYER		(1 << 2)
-#define REQ_STAY_OPEN		(1 << 3)
-#define REQ_STAY_CLOSED		(1 << 4)
-#define REQ_REOPEN		(1 << 5)
+#define REQ_ASK				(1 << 0)
+#define REQ_CONFIRM			(1 << 1)
+#define REQ_PLAYER			(1 << 2)
+#define REQ_STAY_OPEN			(1 << 3)
+#define REQ_STAY_CLOSED			(1 << 4)
+#define REQ_REOPEN			(1 << 5)
 
 
 int getFieldbufferOffsetX_RND(int, int);
@@ -84,6 +83,7 @@ void DrawMaskedBorder_ALL(void);
 void DrawMaskedBorder(int);
 void DrawMaskedBorderToTarget(int);
 void DrawTileCursor(int, int);
+void FreeTileCursorTextures(void);
 
 void SetDrawtoField(int);
 int GetDrawtoField(void);
@@ -94,6 +94,8 @@ void BlitScreenToBitmap(Bitmap *);
 void BackToFront(void);
 void BackToFront_WithFrameDelay(unsigned int);
 
+void CoverScreen(void);
+
 void FadeIn(int);
 void FadeOut(int);
 void FadeSetEnterMenu(void);
@@ -203,6 +205,7 @@ void DrawMiniElementOrWall(int, int, int, int);
 void ShowEnvelope(int);
 void ShowEnvelopeDoor(char *, int);
 void DrawEnvelopeRequestToScreen(int);
+void FreeEnvelopeRequestTextures(void);
 
 void DrawLevel(int);
 void DrawSizedLevel(int, int, int, int, int);
@@ -230,6 +233,14 @@ void UndrawSpecialEditorDoor(void);
 void CreateToolButtons(void);
 void FreeToolButtons(void);
 
+int getEngineElement(int);
+int getDrawingElement(int);
+
+int map_element_RND_to_BD_cave(int);
+int map_element_RND_to_BD_effect(int, int);
+int map_element_BD_to_RND_cave(int);
+int map_element_BD_to_RND_game(int);
+
 int map_element_RND_to_EM_cave(int);
 int map_element_EM_to_RND_cave(int);
 int map_element_EM_to_RND_game(int);
@@ -257,6 +268,7 @@ int el_dir2img(int, int);
 int el2baseimg(int);
 int el2img(int);
 int el2edimg(int);
+int el2edimg_with_frame(int, int *, int *);
 int el2preimg(int);
 int el2panelimg(int);
 int font2baseimg(int);
@@ -274,6 +286,8 @@ int getBeltSwitchElementFromBeltNrAndBeltDirNr(int, int);
 int getBeltSwitchElementFromBeltNrAndBeltDir(int, int);
 
 unsigned int InitRND(int);
+
+void InitGraphicInfo_BD(void);
 void InitGraphicInfo_EM(void);
 
 void PlayMenuSoundExt(int);
@@ -295,6 +309,9 @@ void ResetFontStatus(void);
 
 void SetLevelSetInfo(char *, int);
 
+void ToggleAudioSampleRateIfNeeded(void);
+void ToggleMenuMusicIfNeeded(void);
+void ToggleMenuSoundsIfNeeded(void);
 void ToggleFullscreenIfNeeded(void);
 void ChangeWindowScalingIfNeeded(void);
 void ChangeVsyncModeIfNeeded(void);
